SystemOrganization addCategory: #CryptographyHashing!
SystemOrganization addCategory: #'CryptographyHashing-MD5'!
SystemOrganization addCategory: #'CryptographyHashing-SHA1'!
SystemOrganization addCategory: #'CryptographyHashing-SHA2'!

Object subclass: #HMAC
	instanceVariableNames: 'hashFunction outerPadding innerPadding stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CryptographyHashing'!

!HMAC commentStamp: 'ul 1/19/2021 11:02' prior: 0!
I'm a simple and efficient implementation of HMAC, the keyed-hash message authentication code. See RFC2104 or https://en.wikipedia.org/wiki/HMAC for details.
To create an instance of me, you'll need an instance of HashFunction and a key. The key can be either a ByteString or a ByteArray.
Sending hashMessage: to my initialized instance will calculate the MAC. For example:

((HashFunction newSHA256 hmac
	key: 'key';
	hashMessage: 'The quick brown fox jumps over the lazy dog') hex. "=> 'f7bc83f430538424b13298e6aa6fb143ef4d59a14946175997479dbc2d1a3cd8'"

or

((HMAC on: HashFunction newMD5)
	key: 'key';
	hashMessage: 'The quick brown fox jumps over the lazy dog') hex. "=> '80070713463e7749b90c2dc24911e275'"

My instances can be used multiple times, either with the same key, or by changing the key on the way.

Instance Variables
	hashFunction:		<HashFunction>
	innerPadding		<ByteArray>
	outerPadding:		<ByteArray>
	stream: 			<ReadStream>

hashFunction
	- the HashFunction instance to be used to calculate the HMAC

innerPadding
	- inner padding calculated from the key. see #key:

outerPadding
	- outer padding calculated from the key (see #key:) with additional bytes of size hashFunction hashSize to avoid allocating an extra collection (see #hashMessage:)
	
stream
	- a ReadStream used by #hashMessage:!

!HMAC class methodsFor: 'instance creation' stamp: 'ul 2/21/2018 23:55'!
on: aHashFunction

	^self new
		hashFunction: aHashFunction;
		yourself! !

!HMAC methodsFor: 'accessing' stamp: 'ul 3/16/2020 02:57'!
destroy

	innerPadding ifNotNil: [
		innerPadding destroy.
		innerPadding := nil ].
	outerPadding ifNotNil: [
		outerPadding destroy.
		outerPadding := nil ]! !

!HMAC methodsFor: 'accessing' stamp: 'ul 3/16/2020 02:26'!
digestMessage: aByteArrayOrByteString

	^self hashMessage: aByteArrayOrByteString! !

!HMAC methodsFor: 'accessing' stamp: 'ul 3/16/2020 02:54'!
digestSize

	^hashFunction hashSize! !

!HMAC methodsFor: 'accessing' stamp: 'ul 1/19/2021 10:58'!
hashFunction: aHashFunction

	hashFunction := aHashFunction.
	innerPadding := outerPadding := nil.
	stream := ReadStream basicNew! !

!HMAC methodsFor: 'hashing' stamp: 'ul 1/19/2021 10:59'!
hashMessage: aByteStringOrByteArray

	| paddedInput hash |
	paddedInput := innerPadding grownBy: aByteStringOrByteArray size.
	"By using #replaceFrom:to:with:startingAt:, the underlying primitive will handle ByteString/ByteArray conversion for free."
	paddedInput
		replaceFrom: innerPadding size + 1
		to: paddedInput size
		with: aByteStringOrByteArray
		startingAt: 1.
	stream on: paddedInput.
	hash := ByteArray new: hashFunction hashSize.
	hashFunction hashStream: stream into: hash.
	"By having hashFunction hashSize extra space at the end of outerPadding, allocating an extra collection can be avoided."
	outerPadding
		replaceFrom: outerPadding size - hashFunction hashSize + 1
		to: outerPadding size
		with: hash
		startingAt: 1.
	stream on: outerPadding.
	hashFunction hashStream: stream into: hash.
	^hash! !

!HMAC methodsFor: 'accessing' stamp: 'ul 1/19/2021 10:02'!
key: aStringOrByteArray

	| keyBytes sizeDifference |
	keyBytes := aStringOrByteArray asByteArray.
	sizeDifference := hashFunction blockSize - keyBytes size.
	sizeDifference < 0
		ifTrue: [ keyBytes := hashFunction hashMessage: keyBytes ]
		ifFalse: [
			sizeDifference > 0
				ifTrue: [ keyBytes := keyBytes grownBy: sizeDifference ]
				ifFalse: [ keyBytes == aStringOrByteArray
					ifTrue: [ keyBytes := keyBytes copy ] ] ].
	outerPadding := keyBytes grownBy: hashFunction hashSize.
	innerPadding := keyBytes.
	1 to: keyBytes size do: [ :index |
		outerPadding at: index put: ((outerPadding at: index) bitXor: 16r5C).
		innerPadding at: index put: ((innerPadding at: index) bitXor: 16r36) ]! !

Object subclass: #HashFunction
	instanceVariableNames: 'hash words buffer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CryptographyHashing'!

!HashFunction commentStamp: 'ul 3/2/2020 17:52' prior: 0!
I'm a general purpose cryptographic hash function.

My three instance variables are generally useful for all implementations:
- buffer holds the chunk of input to be hashed.
- hash holds the currently calculated hash value.
- words is the internal state of the hash function.

I also provide a common stream-based interface via the methods #hashStream:, #hashStream:into: and #hashMessage:.
I also support typical chunk creation and padding methods.

Subclasses must implement the following methods:
#copyHashBytesInto: - copies the hash instance variable into the given ByteArray
#hashSize - returns the size of the final hash value in bytes
#initializeHash - sets the inital value of the hash variable before hashing the input
#processBuffer - processes a chunk of the input
#processLastChunkOfSize:totalBytes: - processes the last chunk
!

!HashFunction class methodsFor: 'hashing' stamp: 'ul 2/20/2018 23:28'!
hashMessage: aStringOrByteArray

	^self new hashMessage: aStringOrByteArray! !

!HashFunction class methodsFor: 'hashing' stamp: 'ul 2/20/2018 23:29'!
hashStream: aStream

	^self new hashStream: aStream! !

!HashFunction class methodsFor: 'instance creation' stamp: 'ul 3/16/2020 01:34'!
newMD5

	MD5WithPlugin isPluginAvailable ifTrue: [ ^MD5WithPlugin new ].
	MD5WithCroquetPlugin isPluginAvailable ifTrue: [ ^MD5WithCroquetPlugin new ].
	RGRegister is64BitPlatform ifTrue: [ ^MD5Inlined64 new ].
	^MD5 new! !

!HashFunction class methodsFor: 'instance creation' stamp: 'ul 3/16/2020 01:34'!
newSHA1

	SHA1WithPlugin isPluginAvailable ifTrue: [ ^SHA1WithPlugin new ].
	RGRegister is64BitPlatform ifTrue: [ ^SHA1Inlined64 new ].
	^SHA1 new! !

!HashFunction class methodsFor: 'instance creation' stamp: 'ul 3/16/2020 01:35'!
newSHA224

	SHA224WithSHA2Plugin isPluginAvailable ifTrue: [ ^SHA224WithSHA2Plugin new ].
	SHA224WithPlugin isPluginAvailable ifTrue: [ ^SHA224WithPlugin new ].
	RGRegister is64BitPlatform ifTrue: [ ^SHA224Inlined64 new ].
	^SHA224 new! !

!HashFunction class methodsFor: 'instance creation' stamp: 'ul 3/16/2020 01:35'!
newSHA256

	SHA256WithSHA2Plugin isPluginAvailable ifTrue: [ ^SHA256WithSHA2Plugin new ].
	SHA256WithPlugin isPluginAvailable ifTrue: [ ^SHA256WithPlugin new ].
	RGRegister is64BitPlatform ifTrue: [ ^SHA256Inlined64 new ].
	^SHA256 new! !

!HashFunction class methodsFor: 'instance creation' stamp: 'ul 3/16/2020 01:35'!
newSHA384

	SHA384WithSHA2Plugin isPluginAvailable ifTrue: [ ^SHA384WithSHA2Plugin new ].
	^SHA384 new! !

!HashFunction class methodsFor: 'instance creation' stamp: 'ul 3/16/2020 01:34'!
newSHA512

	SHA512WithSHA2Plugin isPluginAvailable ifTrue: [ ^SHA512WithSHA2Plugin new ].
	^SHA512 new! !

!HashFunction class methodsFor: 'instance creation' stamp: 'ul 3/16/2020 01:35'!
newSHA512p224

	SHA512p224WithSHA2Plugin isPluginAvailable ifTrue: [ ^SHA512p224WithSHA2Plugin new ].
	^SHA512p224 new! !

!HashFunction class methodsFor: 'instance creation' stamp: 'ul 3/16/2020 01:35'!
newSHA512p256

	SHA512p256WithSHA2Plugin isPluginAvailable ifTrue: [ ^SHA512p256WithSHA2Plugin new ].
	^SHA512p256 new! !

!HashFunction methodsFor: 'accessing' stamp: 'ul 2/22/2018 00:07'!
blockSize
	"The number of bytes a chunk hashed by processBuffer consists of; the size of buffer."
	
	self subclassResponsibility! !

!HashFunction methodsFor: 'private' stamp: 'ul 2/19/2018 00:15'!
copyHashBytesInto: aByteArray
	"Copy the calculated hash value into aByteArray"

	self subclassResponsibility! !

!HashFunction methodsFor: 'hashing' stamp: 'ul 2/18/2018 23:47'!
hashMessage: aByteStringOrByteArray

	^self hashStream: aByteStringOrByteArray asByteArray readStream! !

!HashFunction methodsFor: 'accessing' stamp: 'ul 2/18/2018 23:09'!
hashSize
	"The number of bytes the hash consists of"
	
	self subclassResponsibility! !

!HashFunction methodsFor: 'hashing' stamp: 'ul 2/20/2018 22:46'!
hashStream: stream

	| hashBytes |
	hashBytes := ByteArray new: self hashSize.
	self hashStream: stream into: hashBytes.
	^hashBytes! !

!HashFunction methodsFor: 'hashing' stamp: 'ul 3/2/2020 01:33'!
hashStream: stream into: aByteArray

	| chunkSize rounds bufferSize |
	self initializeHash.
	rounds := 0.
	bufferSize := buffer size.
	[
		(chunkSize := stream readInto: buffer startingAt: 1 count: bufferSize) < bufferSize
			ifTrue: [ 
				buffer from: chunkSize + 1 to: bufferSize put: 0.
				true ] 
			ifFalse: [ stream atEnd ] ]
		whileFalse: [
			rounds := rounds + 1.
			self processBuffer ].
	self processLastChunkOfSize: chunkSize totalBytes: rounds * bufferSize + chunkSize.
	^self copyHashBytesInto: aByteArray! !

!HashFunction methodsFor: 'hashing' stamp: 'ul 3/16/2020 01:32'!
hmac

	^HMAC on: self! !

!HashFunction methodsFor: 'private' stamp: 'ul 2/20/2018 21:02'!
initializeHash
	"Initialize the hash variable before hashing a new stream."
	
	self subclassResponsibility! !

!HashFunction methodsFor: 'private' stamp: 'ul 2/19/2018 00:14'!
processBuffer
	"Do a round on buffer."

	self subclassResponsibility! !

!HashFunction methodsFor: 'private' stamp: 'ul 2/19/2018 00:15'!
processLastChunkOfSize: chunkSize totalBytes: totalBytes
	"Do one or more rounds on buffer while applying padding and storing the number of bits processed"

	self subclassResponsibility! !

HashFunction subclass: #MD5
	instanceVariableNames: 'a b c d f'
	classVariableNames: 'ABCDArray IndexArray KArray ShiftArray'
	poolDictionaries: ''
	category: 'CryptographyHashing-MD5'!

!MD5 commentStamp: 'ul 2/20/2018 23:17' prior: 0!
Simple MD5 implementation based on Registers.!

!MD5 class methodsFor: 'class initialization' stamp: 'ul 2/20/2018 02:06'!
initialize

	self
		initializeABCDArray;
		initializeIndexArray;
		initializeKArray;
		initializeShiftArray! !

!MD5 class methodsFor: 'class initialization' stamp: 'ul 2/20/2018 02:05'!
initializeABCDArray
	" self initializeABCDArray "
	
	| newArray |
	newArray := { 16r67452301. 16refcdab89. 16r98badcfe. 16r10325476 } replace: [ :each |
		RGThirtyTwoBitRegister newForPlatformWith: each ].
	ABCDArray := newArray! !

!MD5 class methodsFor: 'class initialization' stamp: 'ul 2/20/2018 02:01'!
initializeIndexArray
	" self initializeIndexArray "
	
	IndexArray := (
		(0 to: 15),
		((16 to: 31) asArray collect: [ :each | 5 * each + 1 \\ 16 ]),
		((32 to: 47) asArray collect: [ :each | 3 * each + 5 \\ 16 ]),
		((48 to: 63) asArray collect: [ :each | 7 * each \\ 16 ])
			+ 1) asByteArray! !

!MD5 class methodsFor: 'class initialization' stamp: 'ul 2/20/2018 01:55'!
initializeKArray
	" self initializeKArray "
	
	| newArray |
	newArray := Array new: 64.
	1 to: 64 do: [ :index |
		newArray at: index put: (RGThirtyTwoBitRegister newForPlatformWith: (index asFloat sin abs timesTwoPower: 32) floor) ].
	KArray := newArray! !

!MD5 class methodsFor: 'class initialization' stamp: 'ul 2/20/2018 01:57'!
initializeShiftArray
	" self initializeShiftArray "
	
	ShiftArray := #[
		7 12 17 22 7 12 17 22 7 12 17 22 7 12 17 22
		5 9 14 20 5 9 14 20 5 9 14 20 5 9 14 20
		4 11 16 23 4 11 16 23 4 11 16 23 4 11 16 23
		6 10 15 21 6 10 15 21 6 10 15 21 6 10 15 21]! !

!MD5 class methodsFor: 'accessing' stamp: 'ul 3/16/2020 03:49'!
oidString

	^'1.2.840.113549.2.5'! !

!MD5 methodsFor: 'accessing' stamp: 'ul 2/22/2018 00:10'!
blockSize

	^64! !

!MD5 methodsFor: 'private' stamp: 'ul 2/20/2018 22:55'!
copyHashBytesInto: aByteArray
	"Copy the calculated hash value into aByteArray"

	1 to: self hashSize // 4 do: [ :index |
		(hash at: index) storeInto: aByteArray at: index * 4 - 3 bigEndian: false ]! !

!MD5 methodsFor: 'accessing' stamp: 'ul 2/20/2018 01:43'!
hashSize
	"The number of bytes the hash consists of"
	
	^16! !

!MD5 methodsFor: 'initialize-release' stamp: 'ul 2/20/2018 02:21'!
initialize

	buffer := ByteArray new: 64.
	a := RGThirtyTwoBitRegister newForPlatform.
	b := a copy.
	c := a copy.
	d := a copy.
	f := a copy.
	words := Array new: 16.
	1 to: 16 do: [ :index | words at: index put: a copy ].
	hash := Array new: 4.
	1 to: 4 do: [ :index | hash at: index put: a copy ]! !

!MD5 methodsFor: 'hashing' stamp: 'ul 2/20/2018 23:19'!
initializeHash

	1 to: 4 do: [ :index |
		(hash at: index) loadFrom: (ABCDArray at: index) ]! !

!MD5 methodsFor: 'private' stamp: 'ul 2/20/2018 22:25'!
processBuffer

	| index t |
	index := 1.
	[ index <= 16 ] whileTrue: [
		(words at: index) loadFrom: buffer at: index * 4 - 3 bigEndian: false.
		index := index + 1 ].

	a loadFrom: (hash at: 1).
	b loadFrom: (hash at: 2).
	c loadFrom: (hash at: 3).
	d loadFrom: (hash at: 4).

	index := 1.
	[ index <= 16 ] whileTrue: [
		f
			loadFrom: d;
			bitXor: c;
			bitAnd: b;
			bitXor: d;
			+= a;
			+= (KArray at: index);
			+= (words at: (IndexArray at: index));
			leftRotateBy: (ShiftArray at: index).
		t := a.
		a := d.
		d := c.
		c := b.
		(b := t)
			loadFrom: c;
			+= f.
		index := index + 1 ].
	
	[ index <= 32 ] whileTrue: [
		f
			loadFrom: c;
			bitXor: b;
			bitAnd: d;
			bitXor: c;
			+= a;
			+= (KArray at: index);
			+= (words at: (IndexArray at: index));
			leftRotateBy: (ShiftArray at: index).
		t := a.
		a := d.
		d := c.
		c := b.
		(b := t)
			loadFrom: c;
			+= f.
		index := index + 1 ].
			
	[ index <= 48 ] whileTrue: [
		f
			loadFrom: d;
			bitXor: c;
			bitXor: b;
			+= a;
			+= (KArray at: index);
			+= (words at: (IndexArray at: index));
			leftRotateBy: (ShiftArray at: index).
		t := a.
		a := d.
		d := c.
		c := b.
		(b := t)
			loadFrom: c;
			+= f.
		index := index + 1 ].

	[ index <= 64 ] whileTrue: [
		f
			loadFrom: d;
			bitInvert;
			bitOr: b;
			bitXor: c;
			+= a;
			+= (KArray at: index);
			+= (words at: (IndexArray at: index));
			leftRotateBy: (ShiftArray at: index).
		t := a.
		a := d.
		d := c.
		c := b.
		(b := t)
			loadFrom: c;
			+= f.
		index := index + 1 ].

	(hash at: 1) += a.
	(hash at: 2) += b.
	(hash at: 3) += c.
	(hash at: 4) += d! !

!MD5 methodsFor: 'private' stamp: 'ul 2/20/2018 22:58'!
processLastChunkOfSize: chunkSize totalBytes: totalBytes

	| size |
	size := chunkSize = 64 
		ifTrue: [ "No place to add padding"
			self processBuffer.
			0 ]
		ifFalse: [ chunkSize ].
	"Add the padding of 1 one bit and 7 zero bits".
	size := size + 1.
	buffer at: size put: 16r80.
	size > 56 ifTrue: [ "Message length won't fit in here."
		self processBuffer.
		size := 0 ].
	buffer 
		from: size + 1
			to: 56
			put: 0;
		unsignedLong64At: 57
			put: totalBytes * 8
			bigEndian: false.
	self processBuffer! !

HashFunction subclass: #OptimizedHashFunction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CryptographyHashing'!

!OptimizedHashFunction commentStamp: 'ul 3/17/2020 23:32' prior: 0!
This subclass of HashFunction only marks classes which use a plugin to speed up hashing or have been inlined, so that they don't use Registers but actual variables instead.!

OptimizedHashFunction subclass: #MD5Inlined64
	instanceVariableNames: ''
	classVariableNames: 'IndexArray KArray ShiftArray'
	poolDictionaries: ''
	category: 'CryptographyHashing-MD5'!

!MD5Inlined64 commentStamp: 'ul 2/20/2018 23:14' prior: 0!
Same as MD5, but the register operations have been inlined, so on 64-bit platforms the performance is better.!

!MD5Inlined64 class methodsFor: 'class initialization' stamp: 'ul 2/21/2018 01:22'!
initialize

	self
		initializeIndexArray;
		initializeKArray;
		initializeShiftArray! !

!MD5Inlined64 class methodsFor: 'class initialization' stamp: 'ul 2/20/2018 02:01'!
initializeIndexArray
	" self initializeIndexArray "
	
	IndexArray := (
		(0 to: 15),
		((16 to: 31) asArray collect: [ :each | 5 * each + 1 \\ 16 ]),
		((32 to: 47) asArray collect: [ :each | 3 * each + 5 \\ 16 ]),
		((48 to: 63) asArray collect: [ :each | 7 * each \\ 16 ])
			+ 1) asByteArray! !

!MD5Inlined64 class methodsFor: 'class initialization' stamp: 'ul 2/20/2018 23:12'!
initializeKArray
	" self initializeKArray "
	
	| newArray |
	newArray := Array new: 64.
	1 to: 64 do: [ :index |
		newArray at: index put: (index asFloat sin abs timesTwoPower: 32) floor ].
	KArray := newArray! !

!MD5Inlined64 class methodsFor: 'class initialization' stamp: 'ul 2/20/2018 01:57'!
initializeShiftArray
	" self initializeShiftArray "
	
	ShiftArray := #[
		7 12 17 22 7 12 17 22 7 12 17 22 7 12 17 22
		5 9 14 20 5 9 14 20 5 9 14 20 5 9 14 20
		4 11 16 23 4 11 16 23 4 11 16 23 4 11 16 23
		6 10 15 21 6 10 15 21 6 10 15 21 6 10 15 21]! !

!MD5Inlined64 class methodsFor: 'accessing' stamp: 'ul 3/16/2020 03:48'!
oidString

	^MD5 oidString! !

!MD5Inlined64 methodsFor: 'accessing' stamp: 'ul 2/22/2018 00:10'!
blockSize

	^64! !

!MD5Inlined64 methodsFor: 'private' stamp: 'ul 2/20/2018 23:03'!
copyHashBytesInto: aByteArray
	"Copy the calculated hash value into aByteArray"

	aByteArray
		unsignedLongAt: 1 put: (hash at: 1) bigEndian: false;
		unsignedLongAt: 5 put: (hash at: 2) bigEndian: false;
		unsignedLongAt: 9 put: (hash at: 3) bigEndian: false;
		unsignedLongAt: 13 put: (hash at: 4) bigEndian: false! !

!MD5Inlined64 methodsFor: 'accessing' stamp: 'ul 2/20/2018 01:43'!
hashSize
	"The number of bytes the hash consists of"
	
	^16! !

!MD5Inlined64 methodsFor: 'initialize-release' stamp: 'ul 2/20/2018 23:02'!
initialize

	buffer := ByteArray new: 64.
	words := Array new: 16.
	hash := Array new: 4! !

!MD5Inlined64 methodsFor: 'hashing' stamp: 'ul 2/21/2018 01:22'!
initializeHash

	hash
		at: 1 put: 16r67452301;
		at: 2 put: 16refcdab89;
		at: 3 put: 16r98badcfe;
		at: 4 put: 16r10325476! !

!MD5Inlined64 methodsFor: 'private' stamp: 'ul 2/20/2018 23:10'!
processBuffer

	| index t a b c d f |
	index := 1.
	[ index <= 16 ] whileTrue: [
		words at: index put: (buffer unsignedLongAt: index * 4 - 3 bigEndian: false).
		index := index + 1 ].

	a := hash at: 1.
	b := hash at: 2.
	c := hash at: 3.
	d := hash at: 4.

	index := 1.
	[ index <= 16 ] whileTrue: [
		f := (((d bitXor: c) bitAnd: b) bitXor: d) + a + (KArray at: index) + (words at: (IndexArray at: index)) bitAnd: 16rFFFFFFFF.
		(t := ShiftArray at: index) >= 29
			ifTrue: [ f := (f bitShift: t - 32) bitOr: (((f bitAnd: 16rF) bitShift: t) bitAnd: 16rFFFFFFFF) ]
			ifFalse: [ f := ((f bitShift: t) bitAnd: 16rFFFFFFFF) bitOr: (f bitShift: t - 32) ].
		t := a.
		a := d.
		d := c.
		c := b.
		b := b + f bitAnd: 16rFFFFFFFF.
		index := index + 1 ].
	
	[ index <= 32 ] whileTrue: [
		f := (((c bitXor: b) bitAnd: d) bitXor: c) + a + (KArray at: index) + (words at: (IndexArray at: index)) bitAnd: 16rFFFFFFFF.
		(t := ShiftArray at: index) >= 29
			ifTrue: [ f := (f bitShift: t - 32) bitOr: (((f bitAnd: 16rF) bitShift: t) bitAnd: 16rFFFFFFFF) ]
			ifFalse: [ f := ((f bitShift: t) bitAnd: 16rFFFFFFFF) bitOr: (f bitShift: t - 32) ].
		t := a.
		a := d.
		d := c.
		c := b.
		b := b + f bitAnd: 16rFFFFFFFF.
		index := index + 1 ].
			
	[ index <= 48 ] whileTrue: [
		f := ((d bitXor: c) bitXor: b) + a + (KArray at: index) + (words at: (IndexArray at: index)) bitAnd: 16rFFFFFFFF.
		(t := ShiftArray at: index) >= 29
			ifTrue: [ f := (f bitShift: t - 32) bitOr: (((f bitAnd: 16rF) bitShift: t) bitAnd: 16rFFFFFFFF) ]
			ifFalse: [ f := ((f bitShift: t) bitAnd: 16rFFFFFFFF) bitOr: (f bitShift: t - 32) ].
		t := a.
		a := d.
		d := c.
		c := b.
		b := b + f bitAnd: 16rFFFFFFFF.
		index := index + 1 ].

	[ index <= 64 ] whileTrue: [
		f := (((d bitXor: 16rFFFFFFFF) bitOr: b) bitXor: c) + a + (KArray at: index) + (words at: (IndexArray at: index)) bitAnd: 16rFFFFFFFF.
		(t := ShiftArray at: index) >= 29
			ifTrue: [ f := (f bitShift: t - 32) bitOr: (((f bitAnd: 16rF) bitShift: t) bitAnd: 16rFFFFFFFF) ]
			ifFalse: [ f := ((f bitShift: t) bitAnd: 16rFFFFFFFF) bitOr: (f bitShift: t - 32) ].
		t := a.
		a := d.
		d := c.
		c := b.
		b := b + f bitAnd: 16rFFFFFFFF.
		index := index + 1 ].
	
	hash
		at: 1 put: ((hash at: 1) + a bitAnd: 16rFFFFFFFF);
		at: 2 put: ((hash at: 2) + b bitAnd: 16rFFFFFFFF);
		at: 3 put: ((hash at: 3) + c bitAnd: 16rFFFFFFFF);
		at: 4 put: ((hash at: 4) + d bitAnd: 16rFFFFFFFF)! !

!MD5Inlined64 methodsFor: 'private' stamp: 'ul 2/20/2018 22:58'!
processLastChunkOfSize: chunkSize totalBytes: totalBytes

	| size |
	size := chunkSize = 64 
		ifTrue: [ "No place to add padding"
			self processBuffer.
			0 ]
		ifFalse: [ chunkSize ].
	"Add the padding of 1 one bit and 7 zero bits".
	size := size + 1.
	buffer at: size put: 16r80.
	size > 56 ifTrue: [ "Message length won't fit in here."
		self processBuffer.
		size := 0 ].
	buffer 
		from: size + 1
			to: 56
			put: 0;
		unsignedLong64At: 57
			put: totalBytes * 8
			bigEndian: false.
	self processBuffer! !

OptimizedHashFunction subclass: #MD5WithCroquetPlugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CryptographyHashing-MD5'!

!MD5WithCroquetPlugin commentStamp: 'ul 2/26/2018 22:18' prior: 0!
Same as MD5, but uses CroquetPlugin's primitiveMD5Transform primitive to process the buffer.!

!MD5WithCroquetPlugin class methodsFor: 'testing' stamp: 'ul 2/26/2018 22:21'!
isPluginAvailable
	"Use primitiveGatherEntropy with an empty ByteArray from CroquetPlugin to test if the plugin is available."

	^self primGatherEntropyInto: #[]! !

!MD5WithCroquetPlugin class methodsFor: 'accessing' stamp: 'ul 3/16/2020 03:48'!
oidString

	^MD5 oidString! !

!MD5WithCroquetPlugin class methodsFor: 'private' stamp: 'ul 2/26/2018 22:21'!
primGatherEntropyInto: aByteArray

	<primitive: 'primitiveGatherEntropy' module: 'CroquetPlugin'>
	^false! !

!MD5WithCroquetPlugin methodsFor: 'accessing' stamp: 'ul 2/22/2018 00:10'!
blockSize

	^64! !

!MD5WithCroquetPlugin methodsFor: 'private' stamp: 'ul 2/20/2018 23:03'!
copyHashBytesInto: aByteArray
	"Copy the calculated hash value into aByteArray"

	aByteArray
		unsignedLongAt: 1 put: (hash at: 1) bigEndian: false;
		unsignedLongAt: 5 put: (hash at: 2) bigEndian: false;
		unsignedLongAt: 9 put: (hash at: 3) bigEndian: false;
		unsignedLongAt: 13 put: (hash at: 4) bigEndian: false! !

!MD5WithCroquetPlugin methodsFor: 'accessing' stamp: 'ul 2/20/2018 01:43'!
hashSize
	"The number of bytes the hash consists of"
	
	^16! !

!MD5WithCroquetPlugin methodsFor: 'initialize-release' stamp: 'ul 2/24/2018 23:46'!
initialize

	buffer := ByteArray new: 64.
	words := WordArray new: 16.
	hash := WordArray new: 4! !

!MD5WithCroquetPlugin methodsFor: 'hashing' stamp: 'ul 2/21/2018 01:22'!
initializeHash

	hash
		at: 1 put: 16r67452301;
		at: 2 put: 16refcdab89;
		at: 3 put: 16r98badcfe;
		at: 4 put: 16r10325476! !

!MD5WithCroquetPlugin methodsFor: 'private' stamp: 'ul 2/24/2018 23:44'!
primitiveMD5Transform: input hash: hash

	<primitive: 'primitiveMD5Transform' module: 'CroquetPlugin'>
	self primitiveFailed! !

!MD5WithCroquetPlugin methodsFor: 'private' stamp: 'ul 2/24/2018 23:50'!
processBuffer

	1 to: 16 do: [ :index |
		words at: index put: (buffer unsignedLongAt: index * 4 - 3 bigEndian: false) ].
	self primitiveMD5Transform: words hash: hash! !

!MD5WithCroquetPlugin methodsFor: 'private' stamp: 'ul 2/20/2018 22:58'!
processLastChunkOfSize: chunkSize totalBytes: totalBytes

	| size |
	size := chunkSize = 64 
		ifTrue: [ "No place to add padding"
			self processBuffer.
			0 ]
		ifFalse: [ chunkSize ].
	"Add the padding of 1 one bit and 7 zero bits".
	size := size + 1.
	buffer at: size put: 16r80.
	size > 56 ifTrue: [ "Message length won't fit in here."
		self processBuffer.
		size := 0 ].
	buffer 
		from: size + 1
			to: 56
			put: 0;
		unsignedLong64At: 57
			put: totalBytes * 8
			bigEndian: false.
	self processBuffer! !

OptimizedHashFunction subclass: #MD5WithPlugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CryptographyHashing-MD5'!

!MD5WithPlugin commentStamp: 'ul 3/6/2020 22:59' prior: 0!
Same as MD5, but uses MD5Plugin's primitives to process the buffer.!

!MD5WithPlugin class methodsFor: 'testing' stamp: 'ul 3/4/2008 00:05'!
isPluginAvailable

	<primitive: 'primitivePluginAvailable' module: 'MD5Plugin'>
	^false! !

!MD5WithPlugin class methodsFor: 'accessing' stamp: 'ul 3/16/2020 03:48'!
oidString

	^MD5 oidString! !

!MD5WithPlugin methodsFor: 'accessing' stamp: 'ul 2/22/2018 00:10'!
blockSize

	^64! !

!MD5WithPlugin methodsFor: 'private' stamp: 'ul 3/6/2020 22:55'!
copyHashBytesInto: aByteArray
	"Copy the calculated hash value into aByteArray"

	self primDecodeState: hash.
	aByteArray copyFrom: hash! !

!MD5WithPlugin methodsFor: 'accessing' stamp: 'ul 2/20/2018 01:43'!
hashSize
	"The number of bytes the hash consists of"
	
	^16! !

!MD5WithPlugin methodsFor: 'initialize-release' stamp: 'ul 3/6/2020 22:58'!
initialize

	buffer := ByteArray new: 64.
	hash := ByteArray new: 16! !

!MD5WithPlugin methodsFor: 'hashing' stamp: 'ul 3/6/2020 22:53'!
initializeHash

	self primInitializeState: hash! !

!MD5WithPlugin methodsFor: 'private' stamp: 'ul 3/7/2008 17:45'!
primDecodeState: s

	<primitive: 'primitiveDecodeState' module: 'MD5Plugin'>
	self primitiveFailed! !

!MD5WithPlugin methodsFor: 'private' stamp: 'ul 3/7/2008 01:17'!
primInitializeState: s

	<primitive: 'primitiveInitializeState' module: 'MD5Plugin'>
	self primitiveFailed! !

!MD5WithPlugin methodsFor: 'private' stamp: 'ul 3/7/2008 01:09'!
primProcessBuffer: aByteArray withState: s

	<primitive: 'primitiveProcessBufferWithState' module: 'MD5Plugin'>
	self primitiveFailed! !

!MD5WithPlugin methodsFor: 'private' stamp: 'ul 3/6/2020 22:54'!
processBuffer

	self primProcessBuffer: buffer withState: hash! !

!MD5WithPlugin methodsFor: 'private' stamp: 'ul 2/20/2018 22:58'!
processLastChunkOfSize: chunkSize totalBytes: totalBytes

	| size |
	size := chunkSize = 64 
		ifTrue: [ "No place to add padding"
			self processBuffer.
			0 ]
		ifFalse: [ chunkSize ].
	"Add the padding of 1 one bit and 7 zero bits".
	size := size + 1.
	buffer at: size put: 16r80.
	size > 56 ifTrue: [ "Message length won't fit in here."
		self processBuffer.
		size := 0 ].
	buffer 
		from: size + 1
			to: 56
			put: 0;
		unsignedLong64At: 57
			put: totalBytes * 8
			bigEndian: false.
	self processBuffer! !

OptimizedHashFunction subclass: #OptimizedHashFunctionWithSHA2Plugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CryptographyHashing-SHA2'!

OptimizedHashFunctionWithSHA2Plugin class
	instanceVariableNames: 'initialHashValues'!

!OptimizedHashFunctionWithSHA2Plugin commentStamp: 'ul 3/16/2020 00:58' prior: 0!
My subclasses use SHA2Plugin's primitives. I implement all the common methods and declare the methods that must be implemented by all subclasses.!

OptimizedHashFunctionWithSHA2Plugin class
	instanceVariableNames: 'initialHashValues'!

!OptimizedHashFunctionWithSHA2Plugin class methodsFor: 'accessing' stamp: 'ul 3/8/2020 23:50'!
initialHashValues

	^initialHashValues! !

!OptimizedHashFunctionWithSHA2Plugin class methodsFor: 'class initialization' stamp: 'ul 3/9/2020 03:04'!
initialize

	self allSubclassesDo: [ :class | class initializeInitialHashValues ]! !

!OptimizedHashFunctionWithSHA2Plugin class methodsFor: 'class initialization' stamp: 'ul 3/9/2020 03:04'!
initializeInitialHashValues
	
	self subclassResponsiblity! !

!OptimizedHashFunctionWithSHA2Plugin class methodsFor: 'testing' stamp: 'ul 3/9/2020 02:10'!
isPluginAvailable

	<primitive: 'primitiveIsPluginAvailable' module: 'SHA2Plugin'>
	^false! !

!OptimizedHashFunctionWithSHA2Plugin methodsFor: 'initialize-release' stamp: 'ul 3/16/2020 00:57'!
initialize

	buffer := ByteArray new: self blockSize.
	"hash is initialized by #initializeHash"! !

!OptimizedHashFunctionWithSHA2Plugin methodsFor: 'private' stamp: 'ul 3/8/2020 23:58'!
initializeHash

	hash := self class initialHashValues shallowCopy! !

!OptimizedHashFunctionWithSHA2Plugin methodsFor: 'private' stamp: 'ul 3/16/2020 00:50'!
primProcessBuffer: aByteArray updatingHash: anWordArrayOrDoubleWordArray

	self subclassResponsiblity! !

!OptimizedHashFunctionWithSHA2Plugin methodsFor: 'private' stamp: 'ul 3/9/2020 01:27'!
processBuffer

	self primProcessBuffer: buffer updatingHash: hash! !

OptimizedHashFunctionWithSHA2Plugin subclass: #SHA256WithSHA2Plugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CryptographyHashing-SHA2'!

!SHA256WithSHA2Plugin commentStamp: 'ul 3/9/2020 03:02' prior: 0!
Same as SHA256, but uses the SHA2Plugin's primitives to process the buffer.!

SHA256WithSHA2Plugin subclass: #SHA224WithSHA2Plugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CryptographyHashing-SHA2'!

!SHA224WithSHA2Plugin commentStamp: 'ul 3/9/2020 03:02' prior: 0!
Same as SHA224, but uses the SHA2Plugin's primitives to process the buffer.!

!SHA224WithSHA2Plugin class methodsFor: 'class initialization' stamp: 'ul 3/9/2020 02:50'!
initializeInitialHashValues
	
	initialHashValues := WordArray withAll: #(
		16rc1059ed8 16r367cd507 16r3070dd17 16rf70e5939 16rffc00b31 16r68581511 16r64f98fa7 16rbefa4fa4)! !

!SHA224WithSHA2Plugin class methodsFor: 'accessing' stamp: 'ul 3/16/2020 03:46'!
oidString

	^SHA224 oidString! !

!SHA224WithSHA2Plugin methodsFor: 'private' stamp: 'ul 3/10/2020 19:08'!
copyHashBytesInto: aByteArray

	self primCopyWords: (hash first: 7) intoBytes: aByteArray! !

!SHA224WithSHA2Plugin methodsFor: 'accessing' stamp: 'ul 2/18/2018 22:32'!
hashSize

	^28! !

!SHA256WithSHA2Plugin class methodsFor: 'class initialization' stamp: 'ul 3/9/2020 02:44'!
initializeInitialHashValues
	
	initialHashValues := WordArray withAll: #(
		16r6a09e667 16rbb67ae85  16r3c6ef372 16ra54ff53a 16r510e527f 16r9b05688c 16r1f83d9ab 16r5be0cd19)! !

!SHA256WithSHA2Plugin class methodsFor: 'accessing' stamp: 'ul 3/16/2020 03:42'!
oidString

	^SHA256 oidString! !

!SHA256WithSHA2Plugin methodsFor: 'accessing' stamp: 'ul 2/22/2018 00:10'!
blockSize

	^64! !

!SHA256WithSHA2Plugin methodsFor: 'private' stamp: 'ul 3/10/2020 19:08'!
copyHashBytesInto: aByteArray

	self primCopyWords: hash intoBytes: aByteArray! !

!SHA256WithSHA2Plugin methodsFor: 'accessing' stamp: 'ul 2/18/2018 22:32'!
hashSize

	^32! !

!SHA256WithSHA2Plugin methodsFor: 'private' stamp: 'ul 3/10/2020 19:09'!
primCopyWords: aDoubleWordArray intoBytes: aByteArray

	<primitive: 'primitiveCopyWordsIntoBytesBigEndian' module: 'SHA2Plugin' error: ec>
	self primitiveFailed! !

!SHA256WithSHA2Plugin methodsFor: 'private' stamp: 'ul 3/16/2020 00:48'!
primProcessBuffer: aByteArray updatingHash: aWordArray

	<primitive: 'primitiveSHA256ProcessBufferUpdatingHash' module: 'SHA2Plugin' error: ec>
	self primitiveFailed! !

!SHA256WithSHA2Plugin methodsFor: 'private' stamp: 'ul 2/18/2018 22:26'!
processLastChunkOfSize: chunkSize totalBytes: totalBytes

	| size |
	size := chunkSize = 64 
		ifTrue: [ "No place to add padding"
			self processBuffer.
			0 ]
		ifFalse: [ chunkSize ].
	"Add the padding of 1 one bit and 7 zero bits".
	size := size + 1.
	buffer at: size put: 16r80.
	size > 56 ifTrue: [ "Message length won't fit in here."
		self processBuffer.
		size := 0 ].
	buffer 
		from: size + 1
			to: 56
			put: 0;
		unsignedLong64At: 57
			put: totalBytes * 8
			bigEndian: true.
	self processBuffer! !

OptimizedHashFunctionWithSHA2Plugin subclass: #SHA512WithSHA2Plugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CryptographyHashing-SHA2'!

!SHA512WithSHA2Plugin commentStamp: 'ul 3/9/2020 03:03' prior: 0!
Same as SHA512, but uses the SHA2Plugin's primitives to process the buffer.!

SHA512WithSHA2Plugin subclass: #SHA384WithSHA2Plugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CryptographyHashing-SHA2'!

!SHA384WithSHA2Plugin commentStamp: 'ul 3/9/2020 03:03' prior: 0!
Same as SHA384, but uses the SHA2Plugin's primitives to process the buffer.!

!SHA384WithSHA2Plugin class methodsFor: 'class initialization' stamp: 'ul 3/9/2020 02:18'!
initializeInitialHashValues
	
	initialHashValues := DoubleWordArray withAll: #(
		16rcbbb9d5dc1059ed8 16r629a292a367cd507 16r9159015a3070dd17 16r152fecd8f70e5939 
		16r67332667ffc00b31 16r8eb44a8768581511 16rdb0c2e0d64f98fa7 16r47b5481dbefa4fa4)! !

!SHA384WithSHA2Plugin class methodsFor: 'accessing' stamp: 'ul 3/16/2020 03:44'!
oidString

	^SHA384 oidString! !

!SHA384WithSHA2Plugin methodsFor: 'private' stamp: 'ul 3/10/2020 19:07'!
copyHashBytesInto: aByteArray

	self primCopyDoubleWords: (hash first: 6) intoBytes: aByteArray! !

!SHA384WithSHA2Plugin methodsFor: 'accessing' stamp: 'ul 2/18/2018 23:42'!
hashSize

	^48! !

!SHA512WithSHA2Plugin class methodsFor: 'class initialization' stamp: 'ul 3/9/2020 02:18'!
initializeInitialHashValues
	
	initialHashValues := DoubleWordArray withAll: #(
		16r6a09e667f3bcc908 16rbb67ae8584caa73b 16r3c6ef372fe94f82b 16ra54ff53a5f1d36f1
		16r510e527fade682d1 16r9b05688c2b3e6c1f 16r1f83d9abfb41bd6b 16r5be0cd19137e2179)! !

!SHA512WithSHA2Plugin class methodsFor: 'accessing' stamp: 'ul 3/16/2020 03:43'!
oidString

	^SHA512 oidString! !

!SHA512WithSHA2Plugin methodsFor: 'accessing' stamp: 'ul 2/22/2018 00:10'!
blockSize

	^128! !

!SHA512WithSHA2Plugin methodsFor: 'private' stamp: 'ul 3/10/2020 19:08'!
copyHashBytesInto: aByteArray

	self primCopyDoubleWords: hash intoBytes: aByteArray! !

!SHA512WithSHA2Plugin methodsFor: 'accessing' stamp: 'ul 2/18/2018 23:08'!
hashSize

	^64! !

!SHA512WithSHA2Plugin methodsFor: 'private' stamp: 'ul 3/10/2020 19:10'!
primCopyDoubleWords: aDoubleWordArray intoBytes: aByteArray

	<primitive: 'primitiveCopyDoubleWordsIntoBytesBigEndian' module: 'SHA2Plugin' error: ec>
	self primitiveFailed! !

!SHA512WithSHA2Plugin methodsFor: 'private' stamp: 'ul 3/9/2020 02:10'!
primProcessBuffer: aByteArray updatingHash: aDoubleWordArray

	<primitive: 'primitiveSHA512ProcessBufferUpdatingHash' module: 'SHA2Plugin' error: ec>
	self primitiveFailed! !

!SHA512WithSHA2Plugin methodsFor: 'private' stamp: 'ul 2/18/2018 23:19'!
processLastChunkOfSize: chunkSize totalBytes: totalBytes

	| size totalBits |
	size := chunkSize = 128 
		ifTrue: [ "No place to add padding"
			self processBuffer.
			0 ]
		ifFalse: [ chunkSize ].
	"Add the padding of 1 one bit and 7 zero bits".
	size := size + 1.
	buffer at: size put: 16r80.
	size > 112 ifTrue: [ "Message length won't fit in here."
		self processBuffer.
		size := 0 ].
	totalBits := totalBytes * 8.
	buffer 
		from: size + 1
			to: 112
			put: 0;
		unsignedLong64At: 113
			put: (totalBits bitShift: -64)
			bigEndian: true;
		unsignedLong64At: 121
			put: (totalBits bitAnd: 16rFFFFFFFFFFFFFFFF)
			bigEndian: true.			
	self processBuffer! !

SHA512WithSHA2Plugin subclass: #SHA512p224WithSHA2Plugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CryptographyHashing-SHA2'!

!SHA512p224WithSHA2Plugin commentStamp: 'ul 3/9/2020 03:03' prior: 0!
Same as SHA512p224, but uses the SHA2Plugin's primitives to process the buffer.!

!SHA512p224WithSHA2Plugin class methodsFor: 'class initialization' stamp: 'ul 3/9/2020 02:56'!
initializeInitialHashValues
	
	initialHashValues := DoubleWordArray withAll: #(
		16r8c3d37c819544da2 16r73e1996689dcd4d6 16r1dfab7ae32ff9c82 16r679dd514582f9fcf
		16r0f6d2b697bd44da8 16r77e36f7304c48942 16r3f9d85a86a1d36c8 16r1112e6ad91d692a1)! !

!SHA512p224WithSHA2Plugin class methodsFor: 'accessing' stamp: 'ul 3/16/2020 03:47'!
oidString

	^SHA512p224 oidString! !

!SHA512p224WithSHA2Plugin methodsFor: 'private' stamp: 'ul 3/10/2020 19:08'!
copyHashBytesInto: aByteArray

	| temp |
	"Target array must large enough to hold 4 double words, but hashSize is just 28, so create a ByteArray of size 32."
	temp := ByteArray new: 32. 
	self primCopyDoubleWords: (hash first: 4) intoBytes: temp.
	aByteArray
		replaceFrom: 1
		to: aByteArray size
		with: temp
		startingAt: 1! !

!SHA512p224WithSHA2Plugin methodsFor: 'accessing' stamp: 'ul 2/19/2018 01:16'!
hashSize

	^28! !

SHA512WithSHA2Plugin subclass: #SHA512p256WithSHA2Plugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CryptographyHashing-SHA2'!

!SHA512p256WithSHA2Plugin commentStamp: 'ul 3/9/2020 03:03' prior: 0!
Same as SHA512p256, but uses the SHA2Plugin's primitives to process the buffer.!

!SHA512p256WithSHA2Plugin class methodsFor: 'class initialization' stamp: 'ul 3/9/2020 02:54'!
initializeInitialHashValues
	
	initialHashValues := DoubleWordArray withAll: #(
		16r22312194fc2bf72c 16r9f555fa3c84c64c2 16r2393b86b6f53b151 16r963877195940eabd
		16r96283ee2a88effe3 16rbe5e1e2553863992 16r2b0199fc2c85b8aa 16r0eb72ddc81c52ca2)! !

!SHA512p256WithSHA2Plugin class methodsFor: 'accessing' stamp: 'ul 3/16/2020 03:47'!
oidString

	^SHA512p256 oidString! !

!SHA512p256WithSHA2Plugin methodsFor: 'private' stamp: 'ul 3/10/2020 19:08'!
copyHashBytesInto: aByteArray

	self primCopyDoubleWords: (hash first: 4) intoBytes: aByteArray.! !

!SHA512p256WithSHA2Plugin methodsFor: 'accessing' stamp: 'ul 2/19/2018 01:11'!
hashSize

	^32! !

OptimizedHashFunction subclass: #SHA1Inlined64
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CryptographyHashing-SHA1'!

!SHA1Inlined64 commentStamp: 'ul 2/21/2018 00:42' prior: 0!
Same as SHA1, but the register operations have been inlined, so on 64-bit platforms the performance is better.!

!SHA1Inlined64 class methodsFor: 'accessing' stamp: 'ul 3/16/2020 03:42'!
oidString

	^SHA1 oidString! !

!SHA1Inlined64 methodsFor: 'accessing' stamp: 'ul 2/22/2018 00:10'!
blockSize

	^64! !

!SHA1Inlined64 methodsFor: 'private' stamp: 'ul 2/21/2018 00:44'!
copyHashBytesInto: aByteArray

	aByteArray
		unsignedLongAt: 1 put: (hash at: 1) bigEndian: true;
		unsignedLongAt: 5 put: (hash at: 2) bigEndian: true;
		unsignedLongAt: 9 put: (hash at: 3) bigEndian: true;
		unsignedLongAt: 13 put: (hash at: 4) bigEndian: true;
		unsignedLongAt: 17 put: (hash at: 5) bigEndian: true		! !

!SHA1Inlined64 methodsFor: 'accessing' stamp: 'ul 2/21/2018 00:09'!
hashSize

	^20! !

!SHA1Inlined64 methodsFor: 'initialize-release' stamp: 'ul 2/21/2018 00:43'!
initialize

	buffer := ByteArray new: 64.
	words := Array new: 80.
	hash := Array new: 5! !

!SHA1Inlined64 methodsFor: 'private' stamp: 'ul 2/21/2018 00:44'!
initializeHash

	hash
		replaceFrom: 1
		to: 5
		with: #(16r67452301 16refcdab89 16r98badcfe 16r10325476 16rc3d2e1f0)
		startingAt: 1! !

!SHA1Inlined64 methodsFor: 'private' stamp: 'ul 2/21/2018 01:08'!
processBuffer

	| index t a b c d e f |
	index := 1.
	[ index <= 16 ] whileTrue: [
		words at: index put: (buffer unsignedLongAt: index * 4 - 3 bigEndian: true).
		index := index + 1 ].

	[ index <= 80 ] whileTrue: [
		t := (((words at: index - 3) bitXor: (words at: index - 8)) bitXor: (words at: index - 14)) bitXor: (words at: index - 16).
		words at: index put: (((t bitShift: 1) bitAnd: 16rFFFFFFFF) bitOr: (t bitShift: -31)).
		index := index + 1 ].

	a := hash at: 1.
	b := hash at: 2.
	c := hash at: 3.
	d := hash at: 4.
	e := hash at: 5.

	index := 1.
	[ index <= 20 ] whileTrue: [
		f := (((c bitXor: d) bitAnd: b) bitXor: d) + e + 16r5a827999 + (words at: index) + ((a bitShift: 5) bitOr: (a bitShift: -27)) bitAnd: 16rFFFFFFFF.
		e := d.
		d := c.
		c := ((b bitAnd: 16r3) bitShift: 30) bitOr: (b bitShift: -2).
		b := a.
		a := f.
		index := index + 1 ].

	[ index <= 40 ] whileTrue: [
		f := ((b bitXor: c) bitXor: d) + e + 16r6ed9eba1 + (words at: index) + ((a bitShift: 5) bitOr: (a bitShift: -27)) bitAnd: 16rFFFFFFFF.
		e := d.
		d := c.
		c := ((b bitAnd: 16r3) bitShift: 30) bitOr: (b bitShift: -2).
		b := a.
		a := f.
		index := index + 1 ].

	[ index <= 60 ] whileTrue: [
		f := (((b bitOr: c) bitAnd: d) bitOr: (b bitAnd: c)) + e + 16r8f1bbcdc + (words at: index) + ((a bitShift: 5) bitOr: (a bitShift: -27)) bitAnd: 16rFFFFFFFF.
		e := d.
		d := c.
		c := ((b bitAnd: 16r3) bitShift: 30) bitOr: (b bitShift: -2).
		b := a.
		a := f.
		index := index + 1 ].

	[ index <= 80 ] whileTrue: [
	f := ((b bitXor: c) bitXor: d) + e + 16rca62c1d6 + (words at: index) + ((a bitShift: 5) bitOr: (a bitShift: -27)) bitAnd: 16rFFFFFFFF.
		e := d.
		d := c.
		c := ((b bitAnd: 16r3) bitShift: 30) bitOr: (b bitShift: -2).
		b := a.
		a := f.
		index := index + 1 ].
	
	hash
		at: 1 put: ((hash at: 1) + a bitAnd: 16rFFFFFFFF);
		at: 2 put: ((hash at: 2) + b bitAnd: 16rFFFFFFFF);
		at: 3 put: ((hash at: 3) + c bitAnd: 16rFFFFFFFF);
		at: 4 put: ((hash at: 4) + d bitAnd: 16rFFFFFFFF);
		at: 5 put: ((hash at: 5) + e bitAnd: 16rFFFFFFFF)! !

!SHA1Inlined64 methodsFor: 'private' stamp: 'ul 2/21/2018 00:24'!
processLastChunkOfSize: chunkSize totalBytes: totalBytes

	| size |
	size := chunkSize = 64 
		ifTrue: [ "No place to add padding"
			self processBuffer.
			0 ]
		ifFalse: [ chunkSize ].
	"Add the padding of 1 one bit and 7 zero bits".
	size := size + 1.
	buffer at: size put: 16r80.
	size > 56 ifTrue: [ "Message length won't fit in here."
		self processBuffer.
		size := 0 ].
	buffer 
		from: size + 1
			to: 56
			put: 0;
		unsignedLong64At: 57
			put: totalBytes * 8
			bigEndian: true.
	self processBuffer! !

OptimizedHashFunction subclass: #SHA1WithPlugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CryptographyHashing-SHA1'!

!SHA1WithPlugin commentStamp: 'ul 2/26/2018 22:19' prior: 0!
Same as SHA1, but uses the DSAPrim plugin's primitives to process the buffer.!

!SHA1WithPlugin class methodsFor: 'testing' stamp: 'ul 2/26/2018 22:12'!
isPluginAvailable
	"Answer true if this platform has primitive support for the Secure Hash Algorithm."

	<primitive: 'primitiveHasSecureHashPrimitive' module: 'DSAPrims'>
	^ false! !

!SHA1WithPlugin class methodsFor: 'accessing' stamp: 'ul 3/16/2020 03:42'!
oidString

	^SHA1 oidString! !

!SHA1WithPlugin methodsFor: 'accessing' stamp: 'ul 2/22/2018 00:10'!
blockSize

	^64! !

!SHA1WithPlugin methodsFor: 'private' stamp: 'ul 2/21/2018 00:44'!
copyHashBytesInto: aByteArray

	aByteArray
		unsignedLongAt: 1 put: (hash at: 1) bigEndian: true;
		unsignedLongAt: 5 put: (hash at: 2) bigEndian: true;
		unsignedLongAt: 9 put: (hash at: 3) bigEndian: true;
		unsignedLongAt: 13 put: (hash at: 4) bigEndian: true;
		unsignedLongAt: 17 put: (hash at: 5) bigEndian: true		! !

!SHA1WithPlugin methodsFor: 'accessing' stamp: 'ul 2/21/2018 00:09'!
hashSize

	^20! !

!SHA1WithPlugin methodsFor: 'initialize-release' stamp: 'ul 2/26/2018 22:10'!
initialize

	buffer := ByteArray new: 64.
	words :=WordArray new: 80.
	hash := WordArray new: 5! !

!SHA1WithPlugin methodsFor: 'private' stamp: 'ul 2/21/2018 00:44'!
initializeHash

	hash
		replaceFrom: 1
		to: 5
		with: #(16r67452301 16refcdab89 16r98badcfe 16r10325476 16rc3d2e1f0)
		startingAt: 1! !

!SHA1WithPlugin methodsFor: 'private' stamp: 'ul 2/26/2018 22:10'!
primExpandBlock: aByteArray into: wordBitmap
	"Expand the given 64-byte buffer into the given WordArray of length 80."

	<primitive: 'primitiveExpandBlock' module: 'DSAPrims'>
	^ self primitiveFailed! !

!SHA1WithPlugin methodsFor: 'private' stamp: 'ul 2/26/2018 22:11'!
primHashBlock: blockBitmap using: workingTotalsBitmap
	"Hash the given block (a Bitmap) of 80 32-bit words, using the given workingTotals."

	<primitive: 'primitiveHashBlock' module: 'DSAPrims'>
	^ self primitiveFailed
! !

!SHA1WithPlugin methodsFor: 'private' stamp: 'ul 2/26/2018 22:11'!
processBuffer

	self
		primExpandBlock: buffer into: words;
		primHashBlock: words using: hash! !

!SHA1WithPlugin methodsFor: 'private' stamp: 'ul 2/21/2018 00:24'!
processLastChunkOfSize: chunkSize totalBytes: totalBytes

	| size |
	size := chunkSize = 64 
		ifTrue: [ "No place to add padding"
			self processBuffer.
			0 ]
		ifFalse: [ chunkSize ].
	"Add the padding of 1 one bit and 7 zero bits".
	size := size + 1.
	buffer at: size put: 16r80.
	size > 56 ifTrue: [ "Message length won't fit in here."
		self processBuffer.
		size := 0 ].
	buffer 
		from: size + 1
			to: 56
			put: 0;
		unsignedLong64At: 57
			put: totalBytes * 8
			bigEndian: true.
	self processBuffer! !

OptimizedHashFunction subclass: #SHA256Inlined64
	instanceVariableNames: ''
	classVariableNames: 'KArray'
	poolDictionaries: ''
	category: 'CryptographyHashing-SHA2'!

!SHA256Inlined64 commentStamp: 'ul 2/20/2018 23:42' prior: 0!
Same as SHA256, but the register operations have been inlined, so on 64-bit platforms the performance is better.!

SHA256Inlined64 subclass: #SHA224Inlined64
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CryptographyHashing-SHA2'!

!SHA224Inlined64 commentStamp: 'ul 2/20/2018 23:42' prior: 0!
Same as SHA224, but the register operations have been inlined, so on 64-bit platforms the performance is better.!

!SHA224Inlined64 class methodsFor: 'accessing' stamp: 'ul 3/16/2020 03:46'!
oidString

	^SHA224 oidString! !

!SHA224Inlined64 methodsFor: 'accessing' stamp: 'ul 2/20/2018 16:18'!
hashSize

	^28! !

!SHA224Inlined64 methodsFor: 'private' stamp: 'ul 2/20/2018 22:53'!
initializeHash

	hash	
		replaceFrom: 1
		to: 8
		with: #(16rc1059ed8 16r367cd507 16r3070dd17 16rf70e5939 16rffc00b31 16r68581511 16r64f98fa7 16rbefa4fa4)
		startingAt: 1! !

!SHA256Inlined64 class methodsFor: 'class initialization' stamp: 'ul 2/20/2018 22:53'!
initialize

	self initializeKArray! !

!SHA256Inlined64 class methodsFor: 'class initialization' stamp: 'ul 2/19/2018 19:52'!
initializeKArray
	" self initializeKArray "
	
	KArray := #(
		16r428a2f98 16r71374491 16rb5c0fbcf 16re9b5dba5 16r3956c25b 16r59f111f1 16r923f82a4 16rab1c5ed5
		16rd807aa98 16r12835b01 16r243185be 16r550c7dc3 16r72be5d74 16r80deb1fe 16r9bdc06a7 16rc19bf174
		16re49b69c1 16refbe4786 16r0fc19dc6 16r240ca1cc 16r2de92c6f 16r4a7484aa 16r5cb0a9dc 16r76f988da
		16r983e5152 16ra831c66d 16rb00327c8 16rbf597fc7 16rc6e00bf3 16rd5a79147 16r06ca6351 16r14292967
		16r27b70a85 16r2e1b2138 16r4d2c6dfc 16r53380d13 16r650a7354 16r766a0abb 16r81c2c92e 16r92722c85
		16ra2bfe8a1 16ra81a664b 16rc24b8b70 16rc76c51a3 16rd192e819 16rd6990624 16rf40e3585 16r106aa070
		16r19a4c116 16r1e376c08 16r2748774c 16r34b0bcb5 16r391c0cb3 16r4ed8aa4a 16r5b9cca4f 16r682e6ff3
		16r748f82ee 16r78a5636f 16r84c87814 16r8cc70208 16r90befffa 16ra4506ceb 16rbef9a3f7 16rc67178f2
	)! !

!SHA256Inlined64 class methodsFor: 'accessing' stamp: 'ul 3/16/2020 03:42'!
oidString

	^SHA256 oidString! !

!SHA256Inlined64 methodsFor: 'accessing' stamp: 'ul 2/22/2018 00:10'!
blockSize

	^64! !

!SHA256Inlined64 methodsFor: 'private' stamp: 'ul 2/19/2018 19:36'!
copyHashBytesInto: aByteArray

	1 to: self hashSize // 4 do: [ :index |
		aByteArray unsignedLongAt: index - 1 * 4 + 1 put: (hash at: index) bigEndian: true ]! !

!SHA256Inlined64 methodsFor: 'accessing' stamp: 'ul 2/18/2018 22:32'!
hashSize

	^32! !

!SHA256Inlined64 methodsFor: 'initialize-release' stamp: 'ul 2/19/2018 20:33'!
initialize

	buffer := ByteArray new: 64.
	words := WordArray new: 64.
	hash := WordArray new: 8! !

!SHA256Inlined64 methodsFor: 'private' stamp: 'ul 2/20/2018 22:53'!
initializeHash

	hash	
		replaceFrom: 1
		to: 8
		with: #(16r6a09e667 16rbb67ae85  16r3c6ef372 16ra54ff53a 16r510e527f 16r9b05688c 16r1f83d9ab 16r5be0cd19)
		startingAt: 1! !

!SHA256Inlined64 methodsFor: 'private' stamp: 'ul 2/21/2018 00:59'!
processBuffer

	| index t s0 s1 a b c d e f g h |
	index := 1.
	[ index <= 16 ] whileTrue: [
		words at: index put: (buffer unsignedLongAt: index * 4 - 3 bigEndian: true).
		index := index + 1 ].
	
	[ index <= 64 ] whileTrue: [	
		t := words at: index - 15.	
		s0 := (t bitShift: -3) bitXor: (
			(((t bitShift: 14) bitAnd: 16rFFFFFFFF) bitOr: (t bitShift: -18)) bitXor: (
				((t bitShift: 25) bitAnd: 16rFFFFFFFF) bitOr: (t bitShift: -7))).
		t := words at: index - 2.
		s1 := (t bitShift: -10) bitXor: (
			(((t bitShift: 13) bitAnd: 16rFFFFFFFF) bitOr: (t bitShift: -19)) bitXor: (
				((t bitShift: 15) bitAnd: 16rFFFFFFFF) bitOr: (t bitShift: -17))).
		words at: index put: (
			(words at: index - 16) + s0 + (words at: index - 7) + s1 bitAnd: 16rFFFFFFFF).
		index := index + 1 ].			

	a := hash at: 1.
	b := hash at: 2.
	c := hash at: 3.
	d := hash at: 4.
	e := hash at: 5.
	f := hash at: 6.
	g := hash at: 7.
	h := hash at: 8.

	index := 1.	
	[ index <= 64 ] whileTrue: [
		"S1 := (e rightrotate 6) xor (e rightrotate 11) xor (e rightrotate 25)"
		s1 := (((e bitShift: 7) bitAnd: 16rFFFFFFFF) bitOr: (e bitShift: -25)) bitXor: (
			(((e bitShift: 21) bitAnd: 16rFFFFFFFF) bitOr: (e bitShift: -11)) bitXor: (
				((e bitShift: 26) bitAnd: 16rFFFFFFFF) bitOr: (e bitShift: -6))).
		"ch := (e and f) xor ((not e) and g)
		temp0
			loadFrom: e;
			bitAnd: f.
		temp1
			loadFrom: e;
			bitInvert;
			bitAnd: g;
			bitXor: temp0;
		 rewritten to: ch := g xor (e and (f xor g))"
		s1 := (((g bitXor: f) bitAnd: e) bitXor: g) + h + s1 +(words at: index) + (KArray at: index) bitAnd: 16rFFFFFFFF.
		"S0 := (a rightrotate 2) xor (a rightrotate 13) xor (a rightrotate 22)"
		s0 := (((a bitShift: 10) bitAnd: 16rFFFFFFFF) bitOr: (a bitShift: -22)) bitXor: (
			(((a bitShift: 19) bitAnd: 16rFFFFFFFF) bitOr: (a bitShift: -13)) bitXor: (
				((a bitAnd: 16r3) bitShift: 30) bitOr: (a bitShift: -2))).
		"maj := (a and b) xor (a and c) xor (b and c)
		temp0
			loadFrom: a;
			bitAnd: b.
		s1
			loadFrom: a;
			bitAnd: c;
			bitXor: temp0.
		temp0
			loadFrom: b;
			bitAnd: c;
			bitXor: s1;
		rewritten to maj := (a and b) or (c and (a or b))"
		s0 := (((a bitOr: b) bitAnd: c) bitOr: (a bitAnd: b)) + s0 bitAnd: 16rFFFFFFFF.
		t := h.
		h := g.
		g := f.
		f := e.
		e := d + s1 bitAnd: 16rFFFFFFFF.
		d := c.
		c := b.
		b := a.
		a := s1 + s0 bitAnd: 16rFFFFFFFF.
		index := index + 1 ].

	hash 
		at: 1 put: ((hash at: 1) + a bitAnd: 16rFFFFFFFF);
		at: 2 put: ((hash at: 2) + b bitAnd: 16rFFFFFFFF);
		at: 3 put: ((hash at: 3) + c bitAnd: 16rFFFFFFFF);
		at: 4 put: ((hash at: 4) + d bitAnd: 16rFFFFFFFF);
		at: 5 put: ((hash at: 5) + e bitAnd: 16rFFFFFFFF);
		at: 6 put: ((hash at: 6) + f bitAnd: 16rFFFFFFFF);
		at: 7 put: ((hash at: 7) + g bitAnd: 16rFFFFFFFF);
		at: 8 put: ((hash at: 8) + h bitAnd: 16rFFFFFFFF)! !

!SHA256Inlined64 methodsFor: 'private' stamp: 'ul 2/18/2018 22:26'!
processLastChunkOfSize: chunkSize totalBytes: totalBytes

	| size |
	size := chunkSize = 64 
		ifTrue: [ "No place to add padding"
			self processBuffer.
			0 ]
		ifFalse: [ chunkSize ].
	"Add the padding of 1 one bit and 7 zero bits".
	size := size + 1.
	buffer at: size put: 16r80.
	size > 56 ifTrue: [ "Message length won't fit in here."
		self processBuffer.
		size := 0 ].
	buffer 
		from: size + 1
			to: 56
			put: 0;
		unsignedLong64At: 57
			put: totalBytes * 8
			bigEndian: true.
	self processBuffer! !

OptimizedHashFunction subclass: #SHA256WithPlugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CryptographyHashing-SHA2'!

!SHA256WithPlugin commentStamp: 'ul 3/9/2020 03:02' prior: 0!
Same as SHA256, but uses the SHA256Plugin's primitives to process the buffer.!

SHA256WithPlugin subclass: #SHA224WithPlugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CryptographyHashing-SHA2'!

!SHA224WithPlugin commentStamp: 'ul 3/16/2020 01:09' prior: 0!
Same as SHA224, but uses the SHA256Plugin's primitives to process the buffer.!

!SHA224WithPlugin class methodsFor: 'accessing' stamp: 'ul 3/16/2020 03:46'!
oidString

	^SHA224 oidString! !

!SHA224WithPlugin methodsFor: 'private' stamp: 'ul 3/7/2020 22:21'!
copyHashBytesInto: aByteArray

	self primFinalHash: hash fromState: hash copy.
	aByteArray copyFrom: hash! !

!SHA224WithPlugin methodsFor: 'accessing' stamp: 'ul 3/7/2020 13:12'!
hashSize

	^28! !

!SHA224WithPlugin methodsFor: 'private' stamp: 'ul 3/16/2020 01:00'!
initializeHash

	hash copyFrom: (Smalltalk endianness = #little
		ifTrue: [ #[216 158 5 193 7 213 124 54 23 221 112 48 57 89 14 247 49 11 192 255 17 21 88 104 167 143 249 100 164 79 250 190] ]
		ifFalse: [ #[193 5 158 216 54 124 213 7 48 112 221 23 247 14 89 57 255 192 11 49 104 88 21 17 100 249 143 167 190 250 79 164] ])! !

!SHA256WithPlugin class methodsFor: 'testing' stamp: 'ul 3/6/2008 03:10'!
isPluginAvailable

	<primitive: 'primitivePluginAvailable' module: 'SHA256Plugin'>
	^false! !

!SHA256WithPlugin class methodsFor: 'accessing' stamp: 'ul 3/16/2020 03:42'!
oidString

	^SHA256 oidString! !

!SHA256WithPlugin methodsFor: 'accessing' stamp: 'ul 2/22/2018 00:10'!
blockSize

	^64! !

!SHA256WithPlugin methodsFor: 'private' stamp: 'ul 3/6/2020 23:05'!
copyHashBytesInto: aByteArray

	self primFinalHash: aByteArray fromState: hash! !

!SHA256WithPlugin methodsFor: 'accessing' stamp: 'ul 2/18/2018 22:32'!
hashSize

	^32! !

!SHA256WithPlugin methodsFor: 'initialize-release' stamp: 'ul 3/6/2020 23:04'!
initialize

	buffer := ByteArray new: 64.
	hash := ByteArray new: 32! !

!SHA256WithPlugin methodsFor: 'private' stamp: 'ul 3/6/2020 23:03'!
initializeHash

	self primInitializeState: hash! !

!SHA256WithPlugin methodsFor: 'private' stamp: 'ul 3/6/2008 22:28'!
primFinalHash: aByteArray fromState: s

	<primitive: 'primitiveFinalHashFromState' module: 'SHA256Plugin'>
	self primitiveFailed! !

!SHA256WithPlugin methodsFor: 'private' stamp: 'ul 3/6/2008 22:27'!
primInitializeState: s

	<primitive: 'primitiveInitializeState' module: 'SHA256Plugin'>
	self primitiveFailed! !

!SHA256WithPlugin methodsFor: 'private' stamp: 'ul 3/6/2008 21:02'!
primProcessBuffer: aByteArray withState: s

	<primitive: 'primitiveProcessBufferWithState' module: 'SHA256Plugin'>
	self primitiveFailed! !

!SHA256WithPlugin methodsFor: 'private' stamp: 'ul 3/6/2020 23:06'!
processBuffer

	self primProcessBuffer: buffer withState: hash! !

!SHA256WithPlugin methodsFor: 'private' stamp: 'ul 2/21/2018 00:24'!
processLastChunkOfSize: chunkSize totalBytes: totalBytes

	| size |
	size := chunkSize = 64 
		ifTrue: [ "No place to add padding"
			self processBuffer.
			0 ]
		ifFalse: [ chunkSize ].
	"Add the padding of 1 one bit and 7 zero bits".
	size := size + 1.
	buffer at: size put: 16r80.
	size > 56 ifTrue: [ "Message length won't fit in here."
		self processBuffer.
		size := 0 ].
	buffer 
		from: size + 1
			to: 56
			put: 0;
		unsignedLong64At: 57
			put: totalBytes * 8
			bigEndian: true.
	self processBuffer! !

HashFunction subclass: #SHA1
	instanceVariableNames: 'a b c d e f temp'
	classVariableNames: 'KArray'
	poolDictionaries: ''
	category: 'CryptographyHashing-SHA1'!

SHA1 class
	instanceVariableNames: 'initialHashValues'!

!SHA1 commentStamp: 'ul 2/21/2018 00:42' prior: 0!
I implement the SHA-1 hash function.

The following document can be helpful to debug the code if needed: https://csrc.nist.gov/csrc/media/projects/cryptographic-standards-and-guidelines/documents/examples/sha1.pdf!

SHA1 class
	instanceVariableNames: 'initialHashValues'!

!SHA1 class methodsFor: 'accessing' stamp: 'ul 2/21/2018 00:07'!
initialHashValues

	^initialHashValues! !

!SHA1 class methodsFor: 'class initialization' stamp: 'ul 2/21/2018 00:19'!
initialize

	self
		initializeInitialHashValues;
		initializeKArray! !

!SHA1 class methodsFor: 'class initialization' stamp: 'ul 2/21/2018 00:08'!
initializeInitialHashValues
	
	| newArray |
	newArray := Array new: 5.
	#(
		16r67452301 16refcdab89 16r98badcfe 16r10325476 16rc3d2e1f0
	) withIndexDo: [ :word :index |
		newArray at: index put: (RGThirtyTwoBitRegister newForPlatformWith: word) ].
	initialHashValues := newArray! !

!SHA1 class methodsFor: 'class initialization' stamp: 'ul 2/21/2018 00:19'!
initializeKArray
	" self initializeKArray "
	
	| newArray |
	newArray := Array new: 4.
	#(
		16r5a827999 16r6ed9eba1 16r8f1bbcdc 16rca62c1d6
	) withIndexDo: [ :word :index |
		newArray at: index put: (RGThirtyTwoBitRegister newForPlatformWith: word) ].
	KArray := newArray! !

!SHA1 class methodsFor: 'accessing' stamp: 'ul 3/16/2020 03:12'!
oidString

	^ '1.3.14.3.2.26'.! !

!SHA1 methodsFor: 'accessing' stamp: 'ul 2/22/2018 00:10'!
blockSize

	^64! !

!SHA1 methodsFor: 'private' stamp: 'ul 2/21/2018 00:09'!
copyHashBytesInto: aByteArray

	1 to: self hashSize // 4 do: [ :index |
		(hash at: index) storeInto: aByteArray at: index * 4 - 3 bigEndian: true ]! !

!SHA1 methodsFor: 'accessing' stamp: 'ul 2/21/2018 00:09'!
hashSize

	^20! !

!SHA1 methodsFor: 'initialize-release' stamp: 'ul 2/21/2018 00:15'!
initialize

	buffer := ByteArray new: 64.
	a := RGThirtyTwoBitRegister newForPlatform.
	b := a copy.
	c := a copy.
	d := a copy.
	e := a copy.
	f := a copy.
	temp := a copy.
	words := Array new: 80.
	1 to: 80 do: [ :index | words at: index put: a copy ].
	hash := Array new: 5.
	1 to: 5 do: [ :index | hash at: index put: a copy ]! !

!SHA1 methodsFor: 'private' stamp: 'ul 2/21/2018 00:25'!
initializeHash

	| initialHashValues |
	initialHashValues := self class initialHashValues.
	1 to: 5 do: [ :index |
		(hash at: index) loadFrom: (initialHashValues at: index) ]! !

!SHA1 methodsFor: 'private' stamp: 'ul 2/21/2018 00:39'!
processBuffer

	| index t k |
	index := 1.
	[ index <= 16 ] whileTrue: [
		(words at: index) loadFrom: buffer at: index * 4 - 3 bigEndian: true.
		index := index + 1 ].
	
	[ index <= 80 ] whileTrue: [
		(words at: index)
			loadFrom: (words at: index - 3);
			bitXor: (words at: index - 8);
			bitXor: (words at: index - 14);
			bitXor: (words at: index - 16);
			leftRotateBy: 1.
		index := index + 1 ].

	a loadFrom: (hash at: 1).
	b loadFrom: (hash at: 2).
	c loadFrom: (hash at: 3).
	d loadFrom: (hash at: 4).
	e loadFrom: (hash at: 5).

	index := 1.
	k := KArray at: 1.
	[ index <= 20 ] whileTrue: [
		f
			loadFrom: c;
			bitXor: d;
			bitAnd: b;
			bitXor: d;
			+= e;
			+= k;
			+= (words at: index).
		temp
			loadFrom: a;
			leftRotateBy: 5;
			+= f.
		t := e.
		e := d.
		d := c.
		(c := b) leftRotateBy: 30.
		b := a.
		a := temp.
		temp := t.
		index := index + 1 ].

	k := KArray at: 2.
	[ index <= 40 ] whileTrue: [
		f
			loadFrom: b;
			bitXor: c;
			bitXor: d;
			+= e;
			+= k;
			+= (words at: index).
		temp
			loadFrom: a;
			leftRotateBy: 5;
			+= f.
		t := e.
		e := d.
		d := c.
		(c := b) leftRotateBy: 30.
		b := a.
		a := temp.
		temp := t.
		index := index + 1 ].

	k := KArray at: 3.
	[ index <= 60 ] whileTrue: [
		temp
			loadFrom: b;
			bitAnd: c.
		f
			loadFrom: b;
			bitOr: c;
			bitAnd: d;
			bitOr: temp;
			+= e;
			+= k;
			+= (words at: index).
		temp
			loadFrom: a;
			leftRotateBy: 5;
			+= f.
		t := e.
		e := d.
		d := c.
		(c := b) leftRotateBy: 30.
		b := a.
		a := temp.
		temp := t.
		index := index + 1 ].

	k := KArray at: 4.
	[ index <= 80 ] whileTrue: [
		f
			loadFrom: b;
			bitXor: c;
			bitXor: d;
			+= e;
			+= k;
			+= (words at: index).
		temp
			loadFrom: a;
			leftRotateBy: 5;
			+= f.
		t := e.
		e := d.
		d := c.
		(c := b) leftRotateBy: 30.
		b := a.
		a := temp.
		temp := t.
		index := index + 1 ].
	
	(hash at: 1) += a.
	(hash at: 2) += b.
	(hash at: 3) += c.
	(hash at: 4) += d.
	(hash at: 5) += e! !

!SHA1 methodsFor: 'private' stamp: 'ul 2/21/2018 00:24'!
processLastChunkOfSize: chunkSize totalBytes: totalBytes

	| size |
	size := chunkSize = 64 
		ifTrue: [ "No place to add padding"
			self processBuffer.
			0 ]
		ifFalse: [ chunkSize ].
	"Add the padding of 1 one bit and 7 zero bits".
	size := size + 1.
	buffer at: size put: 16r80.
	size > 56 ifTrue: [ "Message length won't fit in here."
		self processBuffer.
		size := 0 ].
	buffer 
		from: size + 1
			to: 56
			put: 0;
		unsignedLong64At: 57
			put: totalBytes * 8
			bigEndian: true.
	self processBuffer! !

HashFunction subclass: #SHA2
	instanceVariableNames: 's0 s1 temp0 temp1 a b c d e f g h'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CryptographyHashing-SHA2'!

SHA2 class
	instanceVariableNames: 'initialHashValues'!

!SHA2 commentStamp: 'ul 2/21/2018 00:06' prior: 0!
My subclasses implement the various SHA-2 hash algorithms.

Instance Variables
	buffer:		<ByteArray>
	hash:		<Array of RGThirtyTwoBitRegisters|Array of RGSixtyFourBitRegisters>
	words:		<Array of RGThirtyTwoBitRegisters|Array of RGSixtyFourBitRegisters>
	a:		<RGThirtyTwoBitRegister|RGSixtyFourBitRegisters>
	b:		<RGThirtyTwoBitRegister|RGSixtyFourBitRegisters>
	c:		<RGThirtyTwoBitRegister|RGSixtyFourBitRegisters>
	d:		<RGThirtyTwoBitRegister|RGSixtyFourBitRegisters>
	e:		<RGThirtyTwoBitRegister|RGSixtyFourBitRegisters>
	f:		<RGThirtyTwoBitRegister|RGSixtyFourBitRegisters>
	g:		<RGThirtyTwoBitRegister|RGSixtyFourBitRegisters>
	h:		<RGThirtyTwoBitRegister|RGSixtyFourBitRegisters>
	s0:		<RGThirtyTwoBitRegister|RGSixtyFourBitRegisters>
	s1:		<RGThirtyTwoBitRegister|RGSixtyFourBitRegisters>
	temp0:		<RGThirtyTwoBitRegister|RGSixtyFourBitRegisters>
	temp1:		<RGThirtyTwoBitRegister|RGSixtyFourBitRegisters>

a, b, c, d, e, f, g, h, s0, s1, temp0, temp1 are local variables of #processBuffer. Since they are not temporaries but instance variables, they don't have to be reallocated on every invocation.!

SHA2 class
	instanceVariableNames: 'initialHashValues'!

!SHA2 class methodsFor: 'accessing' stamp: 'ul 2/18/2018 22:34'!
initialHashValues

	^initialHashValues! !

!SHA2 class methodsFor: 'class initialization' stamp: 'ul 3/2/2020 00:24'!
initialize

	self allSubclassesDo: [ :each |
		each
			initializeInitialHashValues
			initializeKArray ]		! !

!SHA2 class methodsFor: 'class initialization' stamp: 'ul 3/2/2020 00:22'!
initializeInitialHashValues

	self subclassResponsiblity! !

!SHA2 class methodsFor: 'class initialization' stamp: 'ul 3/2/2020 00:23'!
initializeKArray

	self subclassResponsiblity! !

!SHA2 methodsFor: 'private' stamp: 'ul 2/20/2018 21:03'!
initializeHash

	| initialHashValues |
	initialHashValues := self class initialHashValues.
	1 to: 8 do: [ :index |
		(hash at: index) loadFrom: (initialHashValues at: index) ]! !

SHA2 subclass: #SHA256
	instanceVariableNames: ''
	classVariableNames: 'KArray'
	poolDictionaries: ''
	category: 'CryptographyHashing-SHA2'!

!SHA256 commentStamp: 'ul 2/20/2018 23:41' prior: 0!
I implement the SHA256 hash function.

The following document can be helpful to debug the code if needed: https://csrc.nist.gov/csrc/media/projects/cryptographic-standards-and-guidelines/documents/examples/sha256.pdf
!

SHA256 subclass: #SHA224
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CryptographyHashing-SHA2'!

!SHA224 commentStamp: 'ul 2/20/2018 23:41' prior: 0!
I implement the SHA224 hash function.

The following document can be helpful to debug the code if needed: https://csrc.nist.gov/csrc/media/projects/cryptographic-standards-and-guidelines/documents/examples/sha224.pdf
!

!SHA224 class methodsFor: 'class initialization' stamp: 'ul 2/20/2018 16:18'!
initializeInitialHashValues
	" self initializeInitialHashValues "
	
	| newArray |
	newArray := Array new: 8.
	#(
		16rc1059ed8 16r367cd507 16r3070dd17 16rf70e5939 16rffc00b31 16r68581511 16r64f98fa7 16rbefa4fa4
	) withIndexDo: [ :word :index |
		newArray at: index put: (RGThirtyTwoBitRegister newForPlatformWith: word) ].
	initialHashValues := newArray! !

!SHA224 class methodsFor: 'accessing' stamp: 'ul 3/16/2020 03:46'!
oidString

	^'2.16.840.1.101.3.4.2.4'! !

!SHA224 methodsFor: 'accessing' stamp: 'ul 2/18/2018 22:32'!
hashSize

	^28! !

!SHA256 class methodsFor: 'accessing' stamp: 'ul 2/18/2018 22:34'!
initialHashValues

	^initialHashValues! !

!SHA256 class methodsFor: 'class initialization' stamp: 'ul 2/18/2018 23:45'!
initializeInitialHashValues
	
	| newArray |
	newArray := Array new: 8.
	#(
		16r6a09e667 16rbb67ae85  16r3c6ef372 16ra54ff53a 16r510e527f 16r9b05688c 16r1f83d9ab 16r5be0cd19
	) withIndexDo: [ :word :index |
		newArray at: index put: (RGThirtyTwoBitRegister newForPlatformWith: word) ].
	initialHashValues := newArray! !

!SHA256 class methodsFor: 'class initialization' stamp: 'ul 2/18/2018 20:38'!
initializeKArray
	" self initializeKArray "
	
	| newArray |
	newArray := Array new: 64.
	#(
		16r428a2f98 16r71374491 16rb5c0fbcf 16re9b5dba5 16r3956c25b 16r59f111f1 16r923f82a4 16rab1c5ed5
		16rd807aa98 16r12835b01 16r243185be 16r550c7dc3 16r72be5d74 16r80deb1fe 16r9bdc06a7 16rc19bf174
		16re49b69c1 16refbe4786 16r0fc19dc6 16r240ca1cc 16r2de92c6f 16r4a7484aa 16r5cb0a9dc 16r76f988da
		16r983e5152 16ra831c66d 16rb00327c8 16rbf597fc7 16rc6e00bf3 16rd5a79147 16r06ca6351 16r14292967
		16r27b70a85 16r2e1b2138 16r4d2c6dfc 16r53380d13 16r650a7354 16r766a0abb 16r81c2c92e 16r92722c85
		16ra2bfe8a1 16ra81a664b 16rc24b8b70 16rc76c51a3 16rd192e819 16rd6990624 16rf40e3585 16r106aa070
		16r19a4c116 16r1e376c08 16r2748774c 16r34b0bcb5 16r391c0cb3 16r4ed8aa4a 16r5b9cca4f 16r682e6ff3
		16r748f82ee 16r78a5636f 16r84c87814 16r8cc70208 16r90befffa 16ra4506ceb 16rbef9a3f7 16rc67178f2
	) withIndexDo: [ :word :index |
		newArray at: index put: (RGThirtyTwoBitRegister newForPlatformWith: word) ].
	KArray := newArray! !

!SHA256 class methodsFor: 'accessing' stamp: 'ul 3/16/2020 03:42'!
oidString

	^'2.16.840.1.101.3.4.2.1'! !

!SHA256 methodsFor: 'accessing' stamp: 'ul 2/22/2018 00:10'!
blockSize

	^64! !

!SHA256 methodsFor: 'private' stamp: 'ul 2/20/2018 22:22'!
copyHashBytesInto: aByteArray

	1 to: self hashSize // 4 do: [ :index |
		(hash at: index) storeInto: aByteArray at: index * 4 - 3 bigEndian: true ]! !

!SHA256 methodsFor: 'accessing' stamp: 'ul 2/18/2018 22:32'!
hashSize

	^32! !

!SHA256 methodsFor: 'initialize-release' stamp: 'ul 2/18/2018 22:10'!
initialize

	buffer := ByteArray new: 64.
	a := RGThirtyTwoBitRegister newForPlatform.
	b := a copy.
	c := a copy.
	d := a copy.
	e := a copy.
	f := a copy.
	g := a copy.
	h := a copy.
	s0 := a copy.
	s1 := a copy.
	temp0 := a copy.
	temp1 := a copy.
	words := Array new: 64.
	1 to: 64 do: [ :index | words at: index put: a copy ].
	hash := Array new: 8.
	1 to: 8 do: [ :index | hash at: index put: a copy ]! !

!SHA256 methodsFor: 'private' stamp: 'ul 2/20/2018 22:27'!
processBuffer

	| t |
	1 to: 16 do: [ :index | 
		(words at: index) loadFrom: buffer at: index * 4 - 3 bigEndian: true ].
	
	17 to: 64 do: [ :index |
		t := words at: index - 15.	
		s0
			loadFrom: t;
			leftRotateBy: -7.
		temp0
			loadFrom: t;
			leftRotateBy: -18;
			bitXor: s0.
		s0
			loadFrom: t;
			>> 3;
			bitXor: temp0.
		t := words at: index - 2.
		s1
			loadFrom: t;
			leftRotateBy: -17.
		temp0
			loadFrom: t;
			leftRotateBy: -19;
			bitXor: s1.
		s1
			loadFrom: t;
			>> 10;
			bitXor: temp0.
		(words at: index)
			loadFrom: (words at: index - 16);
			+= s0;
			+= (words at: index - 7);
			+= s1 ].

	a loadFrom: (hash at: 1).
	b loadFrom: (hash at: 2).
	c loadFrom: (hash at: 3).
	d loadFrom: (hash at: 4).
	e loadFrom: (hash at: 5).
	f loadFrom: (hash at: 6).
	g loadFrom: (hash at: 7).
	h loadFrom: (hash at: 8).

	1 to: 64 do: [ :index |
		"S1 := (e rightrotate 6) xor (e rightrotate 11) xor (e rightrotate 25)"
		s1
			loadFrom: e;
			leftRotateBy: -6.
		temp0
			loadFrom: e;
			leftRotateBy: -11;
			bitXor: s1.
		s1
			loadFrom: e;
			leftRotateBy: -25;
			bitXor: temp0.
		"ch := (e and f) xor ((not e) and g)
		temp0
			loadFrom: e;
			bitAnd: f.
		temp1
			loadFrom: e;
			bitInvert;
			bitAnd: g;
			bitXor: temp0;
		 rewritten to: ch := g xor (e and (f xor g))"
		temp1
			loadFrom: g;
			bitXor: f;
			bitAnd: e;
			bitXor: g;
			"temp1 := h + S1 + ch + k[i] + w[i]"
			+= h;
			+= s1;
			+= (KArray at: index);
			+= (words at: index).
		"S0 := (a rightrotate 2) xor (a rightrotate 13) xor (a rightrotate 22)"
		s0
			loadFrom: a;
			leftRotateBy: -2.
		temp0
			loadFrom: a;
			leftRotateBy: -13;
			bitXor: s0.
		s0
			loadFrom: a;
			leftRotateBy: -22;
			bitXor: temp0.
		"maj := (a and b) xor (a and c) xor (b and c)
		temp0
			loadFrom: a;
			bitAnd: b.
		s1
			loadFrom: a;
			bitAnd: c;
			bitXor: temp0.
		temp0
			loadFrom: b;
			bitAnd: c;
			bitXor: s1;
		rewritten to maj := (a and b) or (c and (a or b))"
		s1
			loadFrom: a;
			bitAnd: b.
		temp0
			loadFrom: a;
			bitOr: b;
			bitAnd: c;
			bitOr: s1;
			"temp2 := S0 + maj"
			+= s0.
		t := h.
		h := g.
		g := f.
		f := e.
		(e := d) += temp1.
		d := c.
		c := b.
		b := a.
		(a := t)
			loadFrom: temp1;
			+= temp0 ].
	(hash at: 1) += a.
	(hash at: 2) += b.
	(hash at: 3) += c.
	(hash at: 4) += d.
	(hash at: 5) += e.
	(hash at: 6) += f.
	(hash at: 7) += g.
	(hash at: 8) += h! !

!SHA256 methodsFor: 'private' stamp: 'ul 2/18/2018 22:26'!
processLastChunkOfSize: chunkSize totalBytes: totalBytes

	| size |
	size := chunkSize = 64 
		ifTrue: [ "No place to add padding"
			self processBuffer.
			0 ]
		ifFalse: [ chunkSize ].
	"Add the padding of 1 one bit and 7 zero bits".
	size := size + 1.
	buffer at: size put: 16r80.
	size > 56 ifTrue: [ "Message length won't fit in here."
		self processBuffer.
		size := 0 ].
	buffer 
		from: size + 1
			to: 56
			put: 0;
		unsignedLong64At: 57
			put: totalBytes * 8
			bigEndian: true.
	self processBuffer! !

SHA2 subclass: #SHA512
	instanceVariableNames: ''
	classVariableNames: 'KArray'
	poolDictionaries: ''
	category: 'CryptographyHashing-SHA2'!

!SHA512 commentStamp: 'ul 2/20/2018 23:41' prior: 0!
I implement the SHA512 hash function.

The following document can be helpful to debug the code if needed: https://csrc.nist.gov/csrc/media/projects/cryptographic-standards-and-guidelines/documents/examples/sha512.pdf
!

SHA512 subclass: #SHA384
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CryptographyHashing-SHA2'!

!SHA384 commentStamp: 'ul 2/20/2018 23:41' prior: 0!
I implement the SHA384 hash function.

The following document can be helpful to debug the code if needed: https://csrc.nist.gov/csrc/media/projects/cryptographic-standards-and-guidelines/documents/examples/sha384.pdf
!

!SHA384 class methodsFor: 'class initialization' stamp: 'ul 2/18/2018 23:43'!
initializeInitialHashValues
	
	| newArray |
	newArray := Array new: 8.
	#(
		16rcbbb9d5dc1059ed8 16r629a292a367cd507 16r9159015a3070dd17 16r152fecd8f70e5939 
		16r67332667ffc00b31 16r8eb44a8768581511 16rdb0c2e0d64f98fa7 16r47b5481dbefa4fa4
	) withIndexDo: [ :word :index |
		newArray at: index put: (RGSixtyFourBitRegister newForPlatformWith: word) ].
	initialHashValues := newArray! !

!SHA384 class methodsFor: 'accessing' stamp: 'ul 3/16/2020 03:44'!
oidString

	^'2.16.840.1.101.3.4.2.2'! !

!SHA384 methodsFor: 'accessing' stamp: 'ul 2/18/2018 23:42'!
hashSize

	^48! !

!SHA512 class methodsFor: 'class initialization' stamp: 'ul 2/20/2018 16:19'!
initializeInitialHashValues
	
	| newArray |
	newArray := Array new: 8.
	#(
		16r6a09e667f3bcc908 16rbb67ae8584caa73b 16r3c6ef372fe94f82b 16ra54ff53a5f1d36f1
		16r510e527fade682d1 16r9b05688c2b3e6c1f 16r1f83d9abfb41bd6b 16r5be0cd19137e2179
	) withIndexDo: [ :word :index |
		newArray at: index put: (RGSixtyFourBitRegister newForPlatformWith: word) ].
	initialHashValues := newArray! !

!SHA512 class methodsFor: 'class initialization' stamp: 'ul 2/18/2018 23:06'!
initializeKArray
	" self initializeKArray "
	
	| newArray |
	newArray := Array new: 80.
	#(
		16r428a2f98d728ae22 16r7137449123ef65cd 16rb5c0fbcfec4d3b2f 16re9b5dba58189dbbc 16r3956c25bf348b538
		16r59f111f1b605d019 16r923f82a4af194f9b 16rab1c5ed5da6d8118 16rd807aa98a3030242 16r12835b0145706fbe 
		16r243185be4ee4b28c 16r550c7dc3d5ffb4e2 16r72be5d74f27b896f 16r80deb1fe3b1696b1 16r9bdc06a725c71235 
		16rc19bf174cf692694 16re49b69c19ef14ad2 16refbe4786384f25e3 16r0fc19dc68b8cd5b5 16r240ca1cc77ac9c65 
		16r2de92c6f592b0275 16r4a7484aa6ea6e483 16r5cb0a9dcbd41fbd4 16r76f988da831153b5 16r983e5152ee66dfab 
		16ra831c66d2db43210 16rb00327c898fb213f 16rbf597fc7beef0ee4 16rc6e00bf33da88fc2 16rd5a79147930aa725 
		16r06ca6351e003826f 16r142929670a0e6e70 16r27b70a8546d22ffc 16r2e1b21385c26c926 16r4d2c6dfc5ac42aed 
		16r53380d139d95b3df 16r650a73548baf63de 16r766a0abb3c77b2a8 16r81c2c92e47edaee6 16r92722c851482353b 
		16ra2bfe8a14cf10364 16ra81a664bbc423001 16rc24b8b70d0f89791 16rc76c51a30654be30 16rd192e819d6ef5218 
		16rd69906245565a910 16rf40e35855771202a 16r106aa07032bbd1b8 16r19a4c116b8d2d0c8 16r1e376c085141ab53 
		16r2748774cdf8eeb99 16r34b0bcb5e19b48a8 16r391c0cb3c5c95a63 16r4ed8aa4ae3418acb 16r5b9cca4f7763e373 
		16r682e6ff3d6b2b8a3 16r748f82ee5defb2fc 16r78a5636f43172f60 16r84c87814a1f0ab72 16r8cc702081a6439ec 
		16r90befffa23631e28 16ra4506cebde82bde9 16rbef9a3f7b2c67915 16rc67178f2e372532b 16rca273eceea26619c 
		16rd186b8c721c0c207 16reada7dd6cde0eb1e 16rf57d4f7fee6ed178 16r06f067aa72176fba 16r0a637dc5a2c898a6 
		16r113f9804bef90dae 16r1b710b35131c471b 16r28db77f523047d84 16r32caab7b40c72493 16r3c9ebe0a15c9bebc 
		16r431d67c49c100d4c 16r4cc5d4becb3e42b6 16r597f299cfc657e2a 16r5fcb6fab3ad6faec 16r6c44198c4a475817
	) withIndexDo: [ :word :index |
		newArray at: index put: (RGSixtyFourBitRegister newForPlatformWith: word) ].
	KArray := newArray! !

!SHA512 class methodsFor: 'accessing' stamp: 'ul 3/16/2020 03:16'!
oidString

	^ '2.16.840.1.101.3.4.2.3'.! !

!SHA512 methodsFor: 'accessing' stamp: 'ul 2/22/2018 00:10'!
blockSize

	^128! !

!SHA512 methodsFor: 'private' stamp: 'ul 2/20/2018 21:08'!
copyHashBytesInto: aByteArray

	1 to: self hashSize // 8 do: [ :index |
		(hash at: index) storeInto: aByteArray at: index * 8 - 7 bigEndian: true ]! !

!SHA512 methodsFor: 'accessing' stamp: 'ul 2/18/2018 23:08'!
hashSize

	^64! !

!SHA512 methodsFor: 'initialize-release' stamp: 'ul 2/18/2018 23:11'!
initialize

	buffer := ByteArray new: 128.
	a := RGSixtyFourBitRegister newForPlatform.
	b := a copy.
	c := a copy.
	d := a copy.
	e := a copy.
	f := a copy.
	g := a copy.
	h := a copy.
	s0 := a copy.
	s1 := a copy.
	temp0 := a copy.
	temp1 := a copy.
	words := Array new: 80.
	1 to: 80 do: [ :index | words at: index put: a copy ].
	hash := Array new: 8.
	1 to: 8 do: [ :index | hash at: index put: a copy ]! !

!SHA512 methodsFor: 'private' stamp: 'ul 2/20/2018 22:27'!
processBuffer

	| t |
	1 to: 16 do: [ :index | 
		(words at: index) loadFrom: buffer at: index * 8 - 7 bigEndian: true ].
	
	17 to: 80 do: [ :index |
		t := words at: index - 15.	
		s0
			loadFrom: t;
			leftRotateBy: -1.
		temp0
			loadFrom: t;
			leftRotateBy: -8;
			bitXor: s0.
		s0
			loadFrom: t;
			>> 7;
			bitXor: temp0.
		t := words at: index - 2.
		s1
			loadFrom: t;
			leftRotateBy: -19.
		temp0
			loadFrom: t;
			leftRotateBy: -61;
			bitXor: s1.
		s1
			loadFrom: t;
			>> 6;
			bitXor: temp0.
		(words at: index)
			loadFrom: (words at: index - 16);
			+= s0;
			+= (words at: index - 7);
			+= s1 ].

	a loadFrom: (hash at: 1).
	b loadFrom: (hash at: 2).
	c loadFrom: (hash at: 3).
	d loadFrom: (hash at: 4).
	e loadFrom: (hash at: 5).
	f loadFrom: (hash at: 6).
	g loadFrom: (hash at: 7).
	h loadFrom: (hash at: 8).

	1 to: 80 do: [ :index |
		"S1 := (e rightrotate 14) xor (e rightrotate 18) xor (e rightrotate 41)"
		s1
			loadFrom: e;
			leftRotateBy: -14.
		temp0
			loadFrom: e;
			leftRotateBy: -18;
			bitXor: s1.
		s1
			loadFrom: e;
			leftRotateBy: -41;
			bitXor: temp0.
		"ch := (e and f) xor ((not e) and g)
		temp0
			loadFrom: e;
			bitAnd: f.
		temp1
			loadFrom: e;
			bitInvert;
			bitAnd: g;
			bitXor: temp0;
		 rewritten to: ch := g xor (e and (f xor g))"
		temp1
			loadFrom: g;
			bitXor: f;
			bitAnd: e;
			bitXor: g;
			"temp1 := h + S1 + ch + k[i] + w[i]"
			+= h;
			+= s1;
			+= (KArray at: index);
			+= (words at: index).
		"S0 := (a rightrotate 28) xor (a rightrotate 34) xor (a rightrotate 39)"
		s0
			loadFrom: a;
			leftRotateBy: -28.
		temp0
			loadFrom: a;
			leftRotateBy: -34;
			bitXor: s0.
		s0
			loadFrom: a;
			leftRotateBy: -39;
			bitXor: temp0.
		"maj := (a and b) xor (a and c) xor (b and c)
		temp0
			loadFrom: a;
			bitAnd: b.
		s1
			loadFrom: a;
			bitAnd: c;
			bitXor: temp0.
		temp0
			loadFrom: b;
			bitAnd: c;
			bitXor: s1;
		rewritten to maj := (a and b) or (c and (a or b))"
		s1
			loadFrom: a;
			bitAnd: b.
		temp0
			loadFrom: a;
			bitOr: b;
			bitAnd: c;
			bitOr: s1;
			"temp2 := S0 + maj"
			+= s0.
		t := h.
		h := g.
		g := f.
		f := e.
		(e := d) += temp1.
		d := c.
		c := b.
		b := a.
		(a := t)
			loadFrom: temp1;
			+= temp0 ].
	(hash at: 1) += a.
	(hash at: 2) += b.
	(hash at: 3) += c.
	(hash at: 4) += d.
	(hash at: 5) += e.
	(hash at: 6) += f.
	(hash at: 7) += g.
	(hash at: 8) += h! !

!SHA512 methodsFor: 'private' stamp: 'ul 2/18/2018 23:19'!
processLastChunkOfSize: chunkSize totalBytes: totalBytes

	| size totalBits |
	size := chunkSize = 128 
		ifTrue: [ "No place to add padding"
			self processBuffer.
			0 ]
		ifFalse: [ chunkSize ].
	"Add the padding of 1 one bit and 7 zero bits".
	size := size + 1.
	buffer at: size put: 16r80.
	size > 112 ifTrue: [ "Message length won't fit in here."
		self processBuffer.
		size := 0 ].
	totalBits := totalBytes * 8.
	buffer 
		from: size + 1
			to: 112
			put: 0;
		unsignedLong64At: 113
			put: (totalBits bitShift: -64)
			bigEndian: true;
		unsignedLong64At: 121
			put: (totalBits bitAnd: 16rFFFFFFFFFFFFFFFF)
			bigEndian: true.			
	self processBuffer! !

SHA512 subclass: #SHA512p224
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CryptographyHashing-SHA2'!

!SHA512p224 commentStamp: 'ul 2/19/2018 01:28' prior: 0!
I implement the SHA512/224 hash function.

The following document can be helpful to debug the code if needed: https://csrc.nist.gov/csrc/media/projects/cryptographic-standards-and-guidelines/documents/examples/sha512_224.pdf
!

!SHA512p224 class methodsFor: 'class initialization' stamp: 'ul 2/19/2018 01:17'!
initializeInitialHashValues
	
	| newArray |
	newArray := Array new: 8.
	#(
		16r8c3d37c819544da2 16r73e1996689dcd4d6 16r1dfab7ae32ff9c82 16r679dd514582f9fcf
		16r0f6d2b697bd44da8 16r77e36f7304c48942 16r3f9d85a86a1d36c8 16r1112e6ad91d692a1
	) withIndexDo: [ :word :index |
		newArray at: index put: (RGSixtyFourBitRegister newForPlatformWith: word) ].
	initialHashValues := newArray! !

!SHA512p224 class methodsFor: 'accessing' stamp: 'ul 3/16/2020 03:47'!
oidString

	^'2.16.840.1.101.3.4.2.5'! !

!SHA512p224 methodsFor: 'private' stamp: 'ul 2/20/2018 21:08'!
copyHashBytesInto: aByteArray

	"Store the last 4 bytes to the beginning, and then copy them to their place at the end."
	(hash at: 4) storeInto: aByteArray at: 1 bigEndian: true.
	aByteArray replaceFrom: 25 to: 28 with: aByteArray startingAt: 1.
	"Fill in the remaining bytes as super does."
	super copyHashBytesInto: aByteArray! !

!SHA512p224 methodsFor: 'accessing' stamp: 'ul 2/19/2018 01:16'!
hashSize

	^28! !

SHA512 subclass: #SHA512p256
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CryptographyHashing-SHA2'!

!SHA512p256 commentStamp: 'ul 2/19/2018 01:28' prior: 0!
I implement the SHA512/256 hash function.

The following document can be helpful to debug the code if needed: https://csrc.nist.gov/csrc/media/projects/cryptographic-standards-and-guidelines/documents/examples/sha512_256.pdf
!

!SHA512p256 class methodsFor: 'class initialization' stamp: 'ul 2/19/2018 01:12'!
initializeInitialHashValues
	
	| newArray |
	newArray := Array new: 8.
	#(
		16r22312194fc2bf72c 16r9f555fa3c84c64c2 16r2393b86b6f53b151 16r963877195940eabd
		16r96283ee2a88effe3 16rbe5e1e2553863992 16r2b0199fc2c85b8aa 16r0eb72ddc81c52ca2
	) withIndexDo: [ :word :index |
		newArray at: index put: (RGSixtyFourBitRegister newForPlatformWith: word) ].
	initialHashValues := newArray! !

!SHA512p256 class methodsFor: 'accessing' stamp: 'ul 3/16/2020 03:47'!
oidString

	^'2.16.840.1.101.3.4.2.6'! !

!SHA512p256 methodsFor: 'accessing' stamp: 'ul 2/19/2018 01:11'!
hashSize

	^32! !
MD5 initialize!
MD5Inlined64 initialize!
OptimizedHashFunctionWithSHA2Plugin initialize!
SHA256Inlined64 initialize!
SHA1 initialize!
SHA2 initialize!
